/* **********************************************************************
 * File: idpl.c	
 * Desc: [I]magen [D]igital [P]rocessing [L]ibrary
 * Auto: Francisco Javier Olmedilla Torralba
 * Vers: 0.0
 *
 * NOTA.- cuando se hace un drag and drop de una ventana histograma a 
 *        una ventana imagen debe existir la ventana de la que se obtuvo
 *        el histograma para que funcione, ya que al cerrar la ventana
 *        original se libera la memoria asociada a la imagen (includo
 *        el histograma). No obstante el histograma se seguir visualizando
 *        ya que previamnte se pint sobre un pixmap.
 *
 * NOTA.- en muchas rutimas necesito de un array colors para almacenar
 * 	  la lut. despues se llamara a la funcion crear imagen que
 * 	  hara una copia de dicho lut. despues libero el array original
 * 	  si declaro el array estatico NO tendria que libararlo, se
 * 	  libera automaticamente al salir de la rutino !!!!!!!!!!!
 * ********************************************************************** */
#include "idpl.h"

/* **********************************************************************
 * Devuelve una imagen contenida en el fichero "file_name" de formato
 * tfe. El formato de estos ficheros es, imgenes monocromo con 256 niveles
 * de gris [0..255]. El tamao es de 320x200 dedicndole un byte a cada
 * pxel.
 * ********************************************************************** */
guchar* idp_image_new_from_tfe_file (const gchar *file_name)
{
    FILE *file;
    guchar* buf = (guchar*) malloc (320 * 200 * sizeof(guchar));

    if (!(file = fopen (file_name, "r")))
    {
	return NULL;
    }
    fgets (buf, 320 * 200, file);
    fclose (file);
    return buf;
}
/* **********************************************************************
 * Inicializa una image. Se har una copia de colors. Por el contrario,
 * no se har una copia de buf.
 * ********************************************************************** */
image_t* idp_image_new (guchar *buf, int width, int height, guint *colors)
{
    image_t *image;
    int i;

    /* imagen: datos, ancho, alto y look up table */
    image = (image_t*) malloc (sizeof(image_t));
    image->buf = buf;
    image->width = width;
    image->height = height;
    if (colors)
	image->cmap = gdk_rgb_cmap_new (colors, 256);
    else
	image->cmap = create_gray_lut ();
    
    /* minimo, maximo, histogramas */
    image->hist_abs = create_absolute_histogram (image);
    image->hist_acu = create_acumulative_histogram (image->hist_abs);
    range_of_grays_levels (&(image->min), &(image->max), image->hist_abs);

    
    /* brillo, constraste, entropa */
    image->bright = bright_of_image (image);
    image->contrast = contrast_of_image (image);
    image->entropy = entropy_of_image (image);

    return image;
}
/* **********************************************************************
 * Inicializa una image. Se har una copia de colors. Por el contrario,
 * no se har una copia de buf.
 * ********************************************************************** */
image_t* idp_image_new_with_hist_abs (guchar *buf, int width, int height, guint *colors, unsigned int *hist_abs)
{
    image_t *image;
    int i;

    /* imagen: datos, ancho, alto y look up table */
    image = (image_t*) malloc (sizeof(image_t));
    image->buf = buf;
    image->width = width;
    image->height = height;
    if (colors)
	image->cmap = gdk_rgb_cmap_new (colors, 256);
    else
	image->cmap = create_gray_lut ();
    
    /* minimo, maximo, histogramas */
    // Esta es la linea que modificaos
    //image->hist_abs = create_absolute_histogram (image);
    image->hist_abs = hist_abs;
    image->hist_acu = create_acumulative_histogram (image->hist_abs);
    range_of_grays_levels (&(image->min), &(image->max), image->hist_abs);

    
    /* brillo, constraste, entropa */
    image->bright = bright_of_image (image);
    image->contrast = contrast_of_image (image);
    image->entropy = entropy_of_image (image);

    return image;
}
/* **********************************************************************
 * Crea una iamgen a partir de un fichero.
 * ********************************************************************** */
image_t* idp_image_new_from_file (const gchar *file_name)
{
    GdkPixbuf *pixbuf = NULL;
    guchar *rgb, *buf;
    int width, height;
    int i,j,fil,fil_rgb,col_rgb;
    gint rowstride, n_channels;

    pixbuf = gdk_pixbuf_new_from_file (file_name, NULL);
    if (!pixbuf)
    {
	/* La imagen ser de tipo .tfe. Si no, se trata de un fichero no valido */
	if (!(buf = idp_image_new_from_tfe_file (file_name)))
	{
	    g_print ("Error al cargar imagen\n");
	    return NULL;
	}
	width = 320;
	height = 200;
    }
    else
    {
	rgb = gdk_pixbuf_get_pixels (pixbuf);		/* image en RGB */
	width = gdk_pixbuf_get_width (pixbuf);
	height = gdk_pixbuf_get_height (pixbuf);
	rowstride = gdk_pixbuf_get_rowstride (pixbuf);
	n_channels = gdk_pixbuf_get_n_channels (pixbuf);
	buf = (guchar*) malloc (width * height * sizeof (guchar));
	for (j = 0, fil = 0, fil_rgb = 0; j < height; j++, fil+=width, fil_rgb+=rowstride)
	    for (i = 0, col_rgb = 0; i < width; i++, col_rgb+=n_channels)
	        buf[(fil+i)] = ROUND(C_R*rgb[(fil_rgb+col_rgb)] + C_G*rgb[(fil_rgb+col_rgb)+1] + C_B*rgb[(fil_rgb+col_rgb)+2]);
	g_object_unref (pixbuf);

    }
    return idp_image_new (buf, width, height, NULL);
}
/* **********************************************************************
 * Crea una nueva imagen a partir de la regin seleccionada de otra
 * ********************************************************************** */
image_t* idp_image_new_subimage	(image_t *image, int x, int y, int width, int height)
{
    guchar *buf, *bufptr;
    int i;

    buf = (guchar*) malloc (abs(width) * abs(height) * sizeof (guchar));

    /* Hacemos que x, y sea la esquina superior izquierda y que width y height sean positivos y no se salgan de rango */
    if (width < 0){
	x += width;
	width = abs (width);
    }
    if (height < 0) {
	y += height;
	height = abs (height);
    }
    if (x < 0) x = 0;
    if (y < 0) y = 0;
    if (width > image->width) width = image->width;
    if (height > image->height) height = image->height;

    /* Se copia la regin seleccionada */
    bufptr = buf;
    for (i = y; i < y+height; i++, bufptr+=width*sizeof(guchar))
	memcpy (bufptr, image->buf+(i*image->width+x), width * sizeof(guchar));

    return idp_image_new (buf, width, height, image->cmap->colors);
}

/* **********************************************************************
 * Crea el negativo de una imagen
 * ********************************************************************** */
image_t* idp_image_new_negative (image_t *image)
{
    gchar *buf;
    guint *colors, tmp;
    image_t* new_image;
    int i;
    
    buf = (gchar*) malloc (image->width * image->height * sizeof (guchar));
    memcpy (buf, image->buf, image->width * image->height * sizeof (guchar));
    colors = (guint*) malloc (256 * sizeof (guint));
    tmp = 255 + (255<<8) +(255<<16);	// OxRRGGBB, en escala de grises RR=GG=BB
    for (i = 0; i < 256; i++)
	colors[i] = tmp - image->cmap->colors[i];
    new_image = idp_image_new (buf, image->width, image->height, colors);
    free (colors);
    return new_image;
}
/* **********************************************************************
 * Crea una nueva imagen en funcin de la transformacin linea que se 
 * espedifica en points. points es un array de 256 elementos. Los valores
 * que toma son:
 *	  -1: punto dentro del tramo
 * 	<>-1: punto extremo del tramo
 * Tanto points[0] como points[255] debern tener valores diferentes a -1.
 * ********************************************************************** */
image_t* idp_image_new_lineal (image_t *image, int *points)
{
    guchar *buf;
    guint *colors;
    image_t *new;
    float m, v;
    int k,i,w;
    guint t;

    colors = (guint*) malloc (256 * sizeof (guint));
    
    for (k = 0, i = 1; i < 256; i++)		/* buscando tramos */
    {
	if (points[i] != -1)			/* tramo: (k, points[k]), (i, points[i]) */
	{
	    m = ((float)points[i] - points[k]) / ((float)i - k);
	    colors[k] = image->cmap->colors[points[k]];
	    colors[i] = image->cmap->colors[points[i]];
	    for (w = k+1, v = m + points[k]; w < i; w++, v += m)
	    {
		t = ROUND(v);
		colors[w] = image->cmap->colors[t];
	    }
	    k = i;
	}
    }
    buf = (guchar*) malloc (image->width * image->height * sizeof (guchar));
    memcpy (buf, image->buf, image->width * image->height * sizeof (guchar));
    new = idp_image_new (buf, image->width, image->height, colors); 
    free (colors);
    return new;
}
/* **********************************************************************
 * Crea una imagen a partir de una dada y de los nuevos valores
 * de brillo y contraste que se desean
 * ********************************************************************** */
image_t *idp_image_new_bright_contrast (image_t *image, float bright, float contrast)
{
    guint *colors;
    guchar *buf;
    image_t *new;
    float a, b, t;
    int i,v;
    
    a = contrast / image->contrast;
    b = bright - (a * image->bright);
    colors = (guint*) malloc (256 * sizeof (guint));
    for (i = 0; i < 256; i++)		/* Vout = A * Vin + B */
    {
	
	t = (a * ((float) (image->cmap->colors[i] & 0xff))) + b;
	v = (guint) t;
	if (v > 255 ) v = 255;
	else if (v < 0) v = 0;
	colors[i] = v + (v<<8) + (v<<16);
    }
    buf = (guchar*) malloc (image->width * image->height * sizeof (guchar));
    memcpy (buf, image->buf, image->width * image->height * sizeof (guchar));
    new = idp_image_new (buf, image->width, image->height, colors); 
    free (colors);
    return new;
}
/* **********************************************************************
 * Dada una imagen se devuelve una nueva tras ecualizar el histograma
 * ********************************************************************** */
image_t *idp_image_new_equalized (image_t *image)
{
    guint *colors;
    guchar *buf;
    image_t *new;
    int i,v;

    colors = (guint*) malloc (256 * sizeof (guint));
    for (i = 0; i < 256; i++)
    {
	/*
	 * Tengo la siguiente duda: no se si se trata de HIST_ACU[i] o si por el contrario
	 * es HIST_ACU[COLORS[i]]
	 * Creo que debe ser HIST_ACU[i]
	 */
	v = ROUND(((float)image->hist_acu[i] / (float)image->hist_acu[255]) * 256) - 1;
	if (v < 0) v = 0;
	if (v > 255) v = 255;
	/* Aqui es donde tengo el problema */
	//colors[i] = v + (v<<8) + (v<<16);
	colors[i] = image->cmap->colors[v];
    }
    buf = (guchar*) malloc (image->width * image->height * sizeof (guchar));
    memcpy (buf, image->buf, image->width * image->height * sizeof (guchar));
    new = idp_image_new (buf, image->width, image->height, colors); 
    free (colors);
    return new;
}
/* **********************************************************************
 * Una nueva imagen a partir de una dada y de un histograma acumulativo.
 * creo que lo hace bien pero NO SE SI SE ESTA HACIENDO BIEN
 * ********************************************************************** */
image_t* idp_image_new_histogram_acu (image_t *image, unsigned int *hist_acu_const)
{
    image_t *new;
    guchar *buf;
    guint *colors;
    unsigned int hist_acu[256];
    float factor;
    unsigned int x, dif_min, dif;
    int i,j, k;

    /* La variable k: nuestra algoritmo buscar el color j-esimo tal que la diferencia entre
     * el histograma_original del color i y el histograma_nuevo del color j sean mnimos.
     * Es decir, lo ms parecido posible. Para encontrar ese 'j' se usa un bucle for y la condicin
     * de haberlo encontrado es que si la nueva diferencia para el j acutal es mayor que la que
     * tena el j anterior, entonces el color buscado es el j anterior. En caso de que ambas
     * diferencia fueran iguales habra que continuar con el bucle for para ver si se llegara
     * a un color que mejore la diferencia. En esta situacin se seguir buscando hasta encontrar
     * dicho color o hasta llegar a 256 (ya no hay ms colores). Entonces, los colors a asignar al
     * i-esimo en adelante no es 255, sino aquel j-esimo color mnimo cuya diferencia del histograma
     * es igual a la de 255. Con la variable k recordamos ese mnimo
     */
    
    /* El primer paso es normaliar el histograma, esto es, hacer que el
     * nmero de pixeles de ambas imgenes se corresponda */
    factor = (float)image->hist_acu[255] / (float)hist_acu_const[255];
    for (i = 0; i < 256; i++)
	hist_acu[i] = ROUND((factor * (float)hist_acu_const[i]));

    /* Se busca la correspondencia Vin -> Vout */
    colors = (guint*) malloc (256 * sizeof (guint));
    for (j = 0, i = 0; i < 256; i++)				/* para cada color i-simo se busca el nuevo color */
    {
	x = image->hist_acu[i];					/* x = su valor en el histograma acumulativo */
	dif_min = INFINITO;
	for (j = j; j < 256; j++)				/* se busca aquel color j-simo cuyo valor en el histograma acumulativo */
	{							/* .. que se pasa por argumento sea lo ms parecido a x */
	    if ((dif = abs(x - hist_acu[j])) < dif_min)
	    {
		dif_min = dif;
		k = j;
	    }
	    else if ((dif = abs(x - hist_acu[j])) == dif_min)
		;
	    else {
		colors[i] = image->cmap->colors[--j];		/* j-1 es el color buscado */
		break;						/* pasamos al siguiene color */
	    }
	}
	if (j == 256)
	{
	    colors[i] = image->cmap->colors[k];
	}
    }
    buf = (guchar*) malloc (image->width * image->height * sizeof (guchar));
    memcpy (buf, image->buf, image->width * image->height * sizeof (guchar));
    new = idp_image_new (buf, image->width, image->height, colors); 
    free (colors);
    return new;
}
/* **********************************************************************
 * Genera una imagen con ruido segn la distribucin de gauss (normal)
 * definida por N(med, dt). Donde med es la media y dt la desviacin
 * tpica. n1 .. n2 definen el intervalo de niveles de gris que va a 
 * tener la imagen. Este intervalo puede ser caulquiera, esto es, puede
 * tomar nmeros negativos, o nmeros mayores que 255, por lo que habr
 * que ajustarlos a 0 .. 255 para poder hacer un representacin grfica
 * de la imagen generada. No obstante los valores que se obtienes sern
 * devueltos a travs de gauss.
 * ********************************************************************** */
image_t* idp_image_new_gauss (int width, int height, double med, double dt, double n1, double n2, double** gauss)
{
    double sum, pend;
    int i, size = width * height;
    char *buf = (guchar*) malloc (size * sizeof (guchar));
   
    /* width*height numeros entre [n1,n2] con distribucin normal
     * N(med, dt)
     */ 
    *gauss = normal (med, dt, size, n1, n2);

    /* Se ajusta [n1..n2] a [0,,255] */
    pend = 255.0 / (n2 - n1);
    for (i = 0; i < size; i++)
	buf[i] = ROUND(pend * ((*gauss)[i] - n1));
    return idp_image_new (buf, width, height, NULL);
}
/* **********************************************************************
 * Dada una imagen, genera una nueva supontieno que se usa un nmero menor
 * de muestras. Para ello se agrupan las muestras de gx x gy  y se calcula
 * su  media, asignando a esas gx x gy muestras el resultado.
 * ********************************************************************** */
image_t* idp_image_new_less_samples (image_t *image, int gx, int gy)
{
    int y, x, i, j;
    int y_lim, x_lim;
    int sum;
    int N = gx * gy;
    guchar *buf;

    buf = (guchar*) malloc (image->width * image->height * sizeof(guchar));
    y_lim = image->height / gy;
    x_lim = image->width / gx;
    y_lim *= gy;
    x_lim *= gx;
    for (y = 0; y < y_lim; y += gy)
	for (x = 0; x < x_lim; x += gx)
	{
	    /* Media para cada grupo de gx x gy pxeles */
	    sum = 0;
	    for (i = y; i < y+gy; i++)
		for (j = x; j < x+gx; j++)
		    sum += image->buf[i*image->width + j];
	    sum = ROUND (sum / N);
	    /* Se asigna la media al grupo de gx x gy pxeles */
	    for (i = y; i < y+gy; i++)
		for (j = x; j < x+gx; j++)
		    buf[i*image->width + j] = sum;
	}
    /* Para el borde derecho */
    if (image->width - x_lim)
    {
	N = gy * (image->width - x_lim);
	for (y = 0; y < y_lim; y+=gy)
	{
	    sum = 0;
	    for (i = y; i < y+gy; i++)
		for (j = x_lim; j < image->width; j++)
		    sum += image->buf[i*image->width + j];
	    sum = ROUND (sum / N);
	    for (i = y; i < y+gy; i++)
		for (j = x_lim; j < image->width; j++)
		    buf[i*image->width + j] = sum;
	}
    }
    /* Para el borde abajo */
    if (image->height - y_lim)
    {
	N = gx * (image->height - y_lim);
	for (x = 0; x < x_lim; x+=gx)
	{
	    sum = 0;
	    for (i = y_lim; i < image->height; i++)
		for (j = x; j < x+gx; j++)
		    sum += image->buf[i*image->width + j];
	    sum = ROUND (sum / N);
	    for (i = y_lim; i < image->height; i++)
		for (j = x; j < x+gx; j++)
		    buf[i*image->width + j] = sum;
	}
    }
		    
    return idp_image_new (buf, image->width, image->height, image->cmap->colors);
}
/* **********************************************************************
 * Dada una imagen devuelve otra simulando que se hayan utilizado nb
 * bits en la digitalizacin. nb bits esta 1..8, y si a una imagen
 * que se le ha aplicado esta operacin con por ejemplo 5 bits, y a la
 * imagen resultante se le aplica esta operacin pero con 7 bits no
 * sufrir una mejora, puesto que la informacin se perdi con la primera
 * de las operaciones aplicadas.
 * ********************************************************************** */
image_t	*idp_image_new_less_bits (image_t *image, int nb)
{
    image_t *new;
    guchar *buf;
    guint *colors;
    int i, j, v, w;
    int n = 8 - nb;

    n = 1 << n;		/* 2^(8-nb) */

    colors = (guint*) malloc (256 * sizeof (guint));
    for (i = 0, j = 255; i < 128; i+=n, j-=n)
    {
	colors[i] = image->cmap->colors[i];
	colors[j] = image->cmap->colors[j];
	for (v = i+1, w = j-1; v < i+n; v++, w--)
	{
	    colors[v] = colors[v-1];
	    colors[w] = colors[w+1];
	}
    }
    buf = (guchar*) malloc (image->width * image->height * sizeof (guchar));
    memcpy (buf, image->buf, image->width * image->height * sizeof (guchar));
    new = idp_image_new (buf, image->width, image->height, colors); 
    free (colors);
    return new;
}
/* **********************************************************************
 * Dada dos imgenes devuelve la diferencia entre ambas en valor
 * absoluto. Las imgenes deben ser de igual dimensiones.
 * ********************************************************************** */
image_t *idp_image_new_difference (image_t *img1, image_t* img2)
{
    guchar *buf;
    int size = img1->width * img1->height;
    int i;
    
    buf = (guchar*) malloc (size * sizeof (guchar));

    for (i = 0; i < size; i++)
	//buf[i] = abs(img1->buf[i] - img2->buf[i]);
	buf[i] = abs((img1->cmap->colors[img1->buf[i]] & 0xff) - (img2->cmap->colors[img2->buf[i]] & 0xff));
    
    return idp_image_new (buf, img1->width, img1->height, NULL);
}
/* **********************************************************************
 * Dada una imagen devuelve la imagen que resulta de aplicarle el 
 * logaritmo. Sera ms eficiente mantener una tabla con la conversin
 * de los logaritmos.
 * ********************************************************************** */
image_t* idp_image_new_logarithm (image_t* image)
{
    int i;
    guchar *buf;
    double _255_div_log_256 = 255.0 / log (256);
    gint tmp;
    guint colors[256];			/* haciendolo estatico (que no static) me ahorro la funcion free(colors) */
    					/* deberia extenderlo al resto de rutinas */
    for (i = 0; i < 256; i++)
    {
	tmp = ROUND (log ((image->cmap->colors[i] & 0xff) + 1) * _255_div_log_256);
	colors[i] = tmp + (tmp << 8) + (tmp << 16);
    }

    buf = (guchar*) malloc (image->width * image->height * sizeof (guchar));
    memcpy (buf, image->buf, image->width * image->height * sizeof (guchar));
    
    return idp_image_new (buf, image->width, image->height, colors);  
}
/* **********************************************************************
 * Dada una imagen devuelve la imagen que resulta de aplicarle el 
 * logaritmo.
 * Para un k = Ln(256) se tiene lo mismo que la funcion idp_image_new_logarithm ()
 * ********************************************************************** */
image_t* idp_image_new_logarithm_k (image_t* image, double k)
{
    int i;
    guchar *buf;
    double c1 = 255.0 / k;
    double c2 = (exp (k) - 1.0) / 255.0;
    gint tmp;
    guint colors[256];			/* haciendolo estatico (que no static) me ahorro la funcion free(colors) */
    					/* deberia extenderlo al resto de rutinas */
    for (i = 0; i < 256; i++)
    {
	tmp = ROUND (log ((image->cmap->colors[i] & 0xff) * c2 + 1) * c1);
	colors[i] = tmp + (tmp << 8) + (tmp << 16);
    }

    buf = (guchar*) malloc (image->width * image->height * sizeof (guchar));
    memcpy (buf, image->buf, image->width * image->height * sizeof (guchar));
  
    return idp_image_new (buf, image->width, image->height, colors);  
}
/* **********************************************************************
 * Dada una imagen devuelve una nueva como resultado de aplicar la
 * funcin logaritmo sobre el histograma acumulativo.
 * Es exactamente igual que idp_image_new_logarithm_k () salvo que 
 * en vez de acturar sobre el COLOR, se actua sobre el histograma_acumulativo[COLOR].
 * ********************************************************************** */
image_t* idp_image_new_logarithm_histogram_k (image_t* image, double k)
{
    int i;
    guchar *buf;
    double c1 = 255.0 / k;
    double c2 = (exp (k) - 1.0) / (image->width * image->height);
    gint tmp;
    guint colors[256];			/* haciendolo estatico (que no static) me ahorro la funcion free(colors) */
    					/* deberia extenderlo al resto de rutinas */
    for (i = 0; i < 256; i++)
    {
	tmp = ROUND (log (image->hist_acu[(image->cmap->colors[i] & 0xff)] * c2 + 1) * c1);
	colors[i] = tmp + (tmp << 8) + (tmp << 16);
    }

    buf = (guchar*) malloc (image->width * image->height * sizeof (guchar));
    memcpy (buf, image->buf, image->width * image->height * sizeof (guchar));
  
    return idp_image_new (buf, image->width, image->height, colors);  
}
/* **********************************************************************
 * Operacin de exponencial (Esta basada en el logaritmo, para ello:
 * negativizamos -> logaritmo -> negativizamos.
 * ********************************************************************** */
image_t* idp_image_new_exponential (image_t* image)
{
    image_t *nega, *loga, *expo;

    nega = idp_image_new_negative (image);
    loga = idp_image_new_logarithm (nega);
    expo = idp_image_new_negative (loga);
    idp_image_free (nega);
    idp_image_free (loga);
    return expo;
}
/* **********************************************************************
 * Operacin de exponencial (Esta basada en el logaritmo, para ello:
 * negativizamos -> logaritmo -> negativizamos.
 * ********************************************************************** */
image_t* idp_image_new_exponential_k (image_t* image, double k)
{
    image_t *nega, *loga, *expo;

    nega = idp_image_new_negative (image);
    loga = idp_image_new_logarithm_k (nega, k);
    expo = idp_image_new_negative (loga);
    idp_image_free (nega);
    idp_image_free (loga);
    return expo;
}
/* **********************************************************************
 * Operacin de exponencial (Esta basada en el logaritmo, para ello:
 * negativizamos -> logaritmo -> negativizamos.
 * ********************************************************************** */
image_t* idp_image_new_exponential_histogram_k (image_t* image, double k)
{
    image_t *nega, *loga, *expo;

    nega = idp_image_new_negative (image);
    loga = idp_image_new_logarithm_histogram_k (nega, k);
    expo = idp_image_new_negative (loga);
    idp_image_free (nega);
    idp_image_free (loga);
    return expo;
}
/* **********************************************************************
 * Dada una imagen, crea una nueva contaminada con ruido gaussiano.
 * Por supuesto las dimensiones del array gauss deben corresponder
 * con las dimensiones de la imagen.
 * ********************************************************************** */
image_t* idp_image_new_add_gauss (image_t *image, double *gauss)
{
    int i;
    int size = image->width * image->height;
    guchar *buf;
    int tmp;

    buf = (guchar*) malloc (size * sizeof (guchar));
    for (i = 0; i < size; i++)		/* para todos los pixels de la imagen */
    {
	tmp = (image->cmap->colors[image->buf[i]] & 0xff);	/* el color REAL (NO EL INDICE AL LUT) del pixel */
	tmp = ROUND((double)tmp + gauss[i]);
	if (tmp < 0) tmp = 0;
	else if (tmp > 255) tmp = 255;
	buf[i] = (guchar)tmp;					/* el nuevo color REAL para ese pixel */
    }
    /* Ya se ha visto que la nueva imagen contien los colores reales para cada pixel, sin embargo nos
     * interesa que el contenido de cada pixel sea un indice al LUT. Lo nico que hay que hacer es que
     * el contenido del LUT se correspondda con el ndice del LUT. Esto se hace pasando NULL a la funcion
     * idp_image_new () */
    return idp_image_new (buf, image->width, image->height, NULL);  
}
/* **********************************************************************
 * Crea una copia de la imagen
 * ********************************************************************** */
image_t* idp_image_copy (image_t *image)
{
    image_t *copy;

    /* Memoira para la imagen */
    copy = (image_t*) malloc (sizeof (image_t));
    memcpy (copy, image, sizeof (image_t));	// Se hace ahora la copia para no borrar los nuevos punteros
    copy->buf = (guchar*) malloc (image->width * image->height * sizeof (guchar));
    copy->hist_abs = (unsigned int*) malloc (sizeof (unsigned int) * 256);
    copy->hist_acu = (unsigned int*) malloc (sizeof (unsigned int) * 256);
    
    /* Se copia los datos */
    memcpy (copy->buf, image->buf, image->width * image->height * sizeof (guchar));
    memcpy (copy->hist_abs, image->hist_abs, 256 * sizeof (unsigned int));
    memcpy (copy->hist_acu, image->hist_acu, 256 * sizeof (unsigned int));
    copy->cmap = gdk_rgb_cmap_new (image->cmap->colors, 256);
	
    return copy;
}
/* **********************************************************************
 * Libera la memoria asociada a una image.
 * ********************************************************************** */
void idp_image_free (image_t *image)
{
    if (!image) return;

    free (image->buf);
    free (image->hist_abs);
    free (image->hist_acu);
    gdk_rgb_cmap_free (image->cmap);
    free (image);
}


/* --------------------------------------------------------------------------------------------------------------------------------------------------- */

/* **********************************************************************
 * Crea una ventana con la imagen
 * ********************************************************************** */
void idp_window_with_image (image_t *image)
{
    GtkWidget *window;
    GtkWidget *vbox;		/* contendr al DA y el StatusBar */
    GtkWidget *darea;
    GtkWidget *statusbar;
    GtkWidget *menu = create_menu(image);
    GtkTargetEntry target_table[] = { 	{TE_TARGET_HIST, TE_FLAGS_HIST, TE_ID_HIST}, 
					{TE_TARGET_IMAGE, TE_FLAGS_IMAGE, TE_ID_IMAGE},
					{TE_TARGET_GAUSS, TE_FLAGS_GAUSS, TE_ID_GAUSS} };
    GtkTargetEntry target_table1[] = { 	{TE_TARGET_IMAGE, TE_FLAGS_IMAGE, TE_ID_IMAGE} };
    image_motion_t *im = (image_motion_t*) malloc (sizeof (image_motion_t));
   
    /* se define la ventana y el drawing area que contendr la image */
    darea = gtk_drawing_area_new ();
    gtk_widget_set_size_request (darea, image->width, image->height);
    statusbar = gtk_statusbar_new ();
    window = gtk_window_new (GTK_WINDOW_TOPLEVEL);
    gtk_window_set_title (GTK_WINDOW(window), "Imagen");
    vbox = gtk_vbox_new (FALSE, 8);
    gtk_container_add (GTK_CONTAINER(vbox), darea);
    gtk_container_add (GTK_CONTAINER(vbox), statusbar);
    gtk_container_add (GTK_CONTAINER(window), vbox);
    gtk_container_set_border_width (GTK_CONTAINER(window), 8);
    //gtk_widget_set_events (window, gtk_widget_get_events (window) | GDK_BUTTON_PRESS_MASK);
    // HAY QUE ACTIVAR LOS EVENTOS ANTES DE MOSTRAR NADA
    gtk_widget_set_events (darea, gtk_widget_get_events (darea) | GDK_POINTER_MOTION_MASK);
    gtk_widget_show_all (window);
    /* las seales a atender */
    im->image = image;
    im->statusbar = statusbar;
    g_signal_connect (G_OBJECT(darea), "expose_event", G_CALLBACK(on_image_expose_event), image);
    g_signal_connect (G_OBJECT(darea), "motion_notify_event", G_CALLBACK(on_image_motion_notify_event), im);
    g_signal_connect (G_OBJECT(window), "destroy", G_CALLBACK(on_image_destroyed), image);
    g_signal_connect (G_OBJECT(window), "destroy", G_CALLBACK(gtk_widget_destroyed), &window);
    g_signal_connect (G_OBJECT(window), "button_press_event", G_CALLBACK(menu_popup), menu);

    /* Drag and Drop como destino: recibira histograma, imgen o ruido gauss. La funcion 
     * on_target_drag_data_received() se encargar de distinguir que es lo que recibe */

    /*  los datos que se reciben, son una copia o una referencia (TENGO PUESTO GDK_ACTION_COPY imagino por tanto que se hace una copia)
     * entonces tendre que liberarlos !!! */
    gtk_drag_dest_set (window, GTK_DEST_DEFAULT_ALL, target_table, 3, GDK_ACTION_COPY);
    g_signal_connect (G_OBJECT(window), "drag_data_received", G_CALLBACK(on_target_drag_data_received), image);

    /* Drag and Drop como fuente: coger la imagen */
    gtk_drag_source_set (window, GDK_BUTTON1_MASK, target_table1, 1, GDK_ACTION_COPY);	/* boton del medio se arrastra la imagen */
    g_signal_connect (G_OBJECT (window), "drag_data_get", G_CALLBACK (on_source_drag_data_get_image), image);
}
/* **********************************************************************
 * Dependiendo de type (1, 0) se pintar el histograma absoluto o el 
 * histograma acumulativo de la imagen. Adems se mostrar ms informacin
 * de la imagen: brillo, contraste, entropia , rango niveles gris, ancho y 
 * alto.
 * ********************************************************************** */
void idp_window_with_histogram2 (image_t *image, int type)
{
    GtkWidget *window, *vbox, *hbox, *da1, *da2;
    GtkWidget *label;
    char str[400];
    GdkPixmap *pixmap;
    gchar *buf = (guchar*) malloc (HEIGHT_LUT * 256 * sizeof (guchar));
    int i, j, max = 0;
    GdkSegment segments[256];
    GtkWidget *statusbar;
    histogram_motion_t *histogram_motion = (histogram_motion_t*) malloc (sizeof (histogram_motion_t));
    unsigned int *histogram = type ? image->hist_abs : image->hist_acu;

    /* mximo valor del histograma */
    for (i = 0; i < 256; i++)
	if (max < histogram[i])
	    max = histogram[i];
    
    /* se escala el histograma a las dimensiones de la ventana (size x size) */
    for (i = 0; i < 256; i++) {
	segments[i].x1 = i;
	segments[i].y1 = 255;
	segments[i].x2 = i;
	segments[i].y2 = (unsigned int)(255.0 - (float)histogram[i] * 255.0 / (float)max);
    }

    /* el LUT */
    for (i = 0; i < HEIGHT_LUT; i++)
	for (j = 0; j < 256; j++)
	    buf[i*256+j] = image->cmap->colors[j];

    hbox = gtk_hbox_new (FALSE, 8);			/* caja horizontal */
    sprintf (str, "Brillo:%g Contraste:%g\nEntropia:%2g\nWidth:%d Height:%d\nNiveles Gris:[%d, %d]", \
	     image->bright, image->contrast, image->entropy, image->width, image->height, image->min, image->max);
    label = gtk_label_new (str);
    gtk_box_pack_start (GTK_BOX (hbox), label, TRUE, TRUE, 0);

    da1 = gtk_drawing_area_new ();			/* DA para histograma */
    gtk_widget_set_size_request (da1, 256, 256);
    
    da2 = gtk_drawing_area_new ();			/* DA para LUT */
    gtk_widget_set_size_request (da2, 256, HEIGHT_LUT);

    statusbar = gtk_statusbar_new ();			/* BARRA DE ESTADO */
	
    vbox = gtk_vbox_new (FALSE, 8);			/* caja vertica */
    gtk_container_set_border_width (GTK_CONTAINER (vbox), 8);
    gtk_box_pack_start (GTK_BOX (vbox), hbox, TRUE, TRUE, 0);
    gtk_box_pack_start (GTK_BOX (vbox), da1, TRUE, TRUE, 0);
    gtk_box_pack_start (GTK_BOX (vbox), da2, TRUE, TRUE, 0);
    gtk_box_pack_start (GTK_BOX (vbox), statusbar, TRUE, TRUE, 0);

    window = gtk_window_new (GTK_WINDOW_TOPLEVEL);
    gtk_window_set_title (GTK_WINDOW(window), "Histrograma");	/* la ventana */
    gtk_container_add (GTK_CONTAINER(window), vbox);
    //gtk_container_set_border_width (GTK_CONTAINER(window), 8);
    gtk_widget_set_events (da1, gtk_widget_get_events (da1) | GDK_POINTER_MOTION_MASK);
    gtk_widget_show_all (window);
	     
    pixmap = gdk_pixmap_new (da1->window, 255, 255, -1);	/* pixmap para el histograma */
    gdk_draw_rectangle (pixmap, da1->style->black_gc, TRUE, 0, 0, 255, 255);
    gdk_draw_segments (pixmap, da1->style->white_gc, segments, 255);

    /* las seeles */
    histogram_motion->image = image;
    histogram_motion->type = type;
    histogram_motion->statusbar = statusbar;
    g_signal_connect (G_OBJECT(da1), "motion_notify_event", G_CALLBACK(on_histogram_motion_notify_event), histogram_motion);
    g_signal_connect (G_OBJECT(da1), "expose_event", G_CALLBACK(on_histogram_expose_event), pixmap);
    g_signal_connect (G_OBJECT(da2), "expose_event", G_CALLBACK(on_lut_expose_event), buf);

    /* Configurando el Drag And Drop: fuente */

    if (!type)		/* para los histogramas ACUMULATIVOS */
    {
	GtkTargetEntry target_table[] = { {TE_TARGET_HIST, TE_FLAGS_HIST, TE_ID_HIST} };
	gtk_drag_source_set (window, GDK_BUTTON1_MASK, target_table, 1, GDK_ACTION_COPY);
	g_signal_connect (G_OBJECT (window), "drag_data_get", G_CALLBACK (on_source_drag_data_get_histogram), histogram);
    }

}
/* **********************************************************************
 * Crea una ventana con el histograma
 * ********************************************************************** */
void idp_window_with_histogram (unsigned int *histogram)
{
    int size = 256;
    GtkWidget *window;
    GtkWidget *darea;
    GdkPixmap *pixmap;
    GdkSegment segments[size];
    unsigned int max = 0, i;
   
    /* mximo valor del histograma */
    for (i = 0; i < size; i++)
	if (max < histogram[i])
	    max = histogram[i];
    
    /* se escala el histograma a las dimensiones de la ventana (size x size) */
    for (i = 0; i < size; i++) {
	segments[i].x1 = i;
	segments[i].y1 = size-1;
	segments[i].x2 = i;
	segments[i].y2 = (unsigned int)((float)(size-1) - (float)histogram[i] * (float)(size-1) / (float)max);
    }
   
    /* se define la ventana y el drawing area que contendr el pixmap */
    darea = gtk_drawing_area_new ();
    gtk_widget_set_size_request (darea, size, size);
    window = gtk_window_new (GTK_WINDOW_TOPLEVEL);
    gtk_window_set_title (GTK_WINDOW(window), "Histrograma");
    gtk_container_add (GTK_CONTAINER(window), darea);
    gtk_container_set_border_width (GTK_CONTAINER(window), 8);
    gtk_widget_show_all (window);

    /* se define el pixmap */
    pixmap = gdk_pixmap_new (darea->window, size, size, -1);
    gdk_draw_rectangle (pixmap, darea->style->black_gc, TRUE, 0, 0, size, size);
    gdk_draw_segments (pixmap, darea->style->white_gc, segments, size);
    
    g_signal_connect (G_OBJECT(darea), "expose_event", G_CALLBACK(on_histogram_expose_event), pixmap);
    /* se libera el pixmap cuando se cierra la ventana? */
    g_signal_connect (G_OBJECT(window), "destroy", G_CALLBACK(gtk_widget_destroyed), &window);
}
/* **********************************************************************
 * Crea una ventana con el Look Up Table que se le pasa. colors es un
 * array de 256 elementos. Sus ndices se corresponden con el color, esto
 * es, el colors[5] se refiere al color n 5. El contenido del array
 * es el color. Por ejemplo el colors[5] = 0xrrggbb = 0x080808 almacena
 * una intensidad de gris de 8. Ntese que lo que se almacena son los
 * valores de R G y B, y que para simular una escada de grises de hace
 * R = G = B.
 * ********************************************************************** */
void idp_window_with_lut (guint *colors)
{
    GtkWidget *window;
    GtkWidget *darea;
    guchar *buf;
    int i,j;
   
    buf = (guchar*) malloc (256 * WIDTH_LUT * sizeof (guchar));
    for (i = 0; i < 256; i++)
	for (j = 0; j < WIDTH_LUT; j++)
	    buf[i*WIDTH_LUT+j] = colors[i];
    /* se define la ventana y el drawing area que contendr la image */
    darea = gtk_drawing_area_new ();
    gtk_widget_set_size_request (darea, WIDTH_LUT, 256);
    window = gtk_window_new (GTK_WINDOW_TOPLEVEL);
    gtk_window_set_title (GTK_WINDOW(window), "LUT");
    gtk_container_add (GTK_CONTAINER(window), darea);
    gtk_container_set_border_width (GTK_CONTAINER(window), 8);
    gtk_widget_show_all (window);
    g_signal_connect (G_OBJECT(darea), "expose_event", G_CALLBACK(on_lut_expose_event), buf);
    g_signal_connect (G_OBJECT(window), "destroy", G_CALLBACK(on_lut_destroyed), buf);
    g_signal_connect (G_OBJECT(window), "destroy", G_CALLBACK(gtk_widget_destroyed), &window);
}
/* **********************************************************************
 * Crear una ventana con un pixmap sobre el que se ha pintado una imagen y unos puntos
 * rojos. Estos puntos representan los pixeles que han experimentado
 * cambio. El uso de esta funcin solo tiene sentido cuando se quiere
 * representar los pixeles que han cambiado en una imagen I1 con respecto
 * a una imagen I2 de la misma escena pero desplazada en el tiempo. image
 * es la imagen I1. changes es una array de dimensiones igual al de la 
 * imagen que indica los puntos rojos. Lo ms normal es que este array 
 * se consiga a partir de la funcin pixel_changes().
 * ********************************************************************** */
void  idp_window_with_image_and_changes (image_t *image, char* changes)
{
    GtkWidget *window;
    GtkWidget *darea;
    GdkPixmap *pixmap;
    int i, j, count;
    GdkGC *gc;
    GdkColor color = {0, 0xffff, 0, 0};		/* Rojo */

    /* se define la ventana y el drawing area que contendr el pixmap */
    darea = gtk_drawing_area_new ();
    gtk_widget_set_size_request (darea, image->width, image->height);
    window = gtk_window_new (GTK_WINDOW_TOPLEVEL);
    gtk_window_set_title (GTK_WINDOW(window), "Imagen/Cambios");
    gtk_container_add (GTK_CONTAINER(window), darea);
    gtk_container_set_border_width (GTK_CONTAINER(window), 8);
    gtk_widget_show_all (window);

    /* se define el pixmap */
    pixmap = gdk_pixmap_new (darea->window, image->width, image->height, -1);
   
    //g_print ("se ha definido el pixmap\n");
    /* se pinta la imagen sobre el pixmap */
    gdk_draw_indexed_image (
	    pixmap,
	    darea->style->fg_gc[GTK_STATE_NORMAL],
	    0,
	    0,
	    image->width, 
	    image->height,
	    GDK_RGB_DITHER_MAX,
	    image->buf,
	    image->width,
	    image->cmap);
    //g_print ("se ha pintado la imagen original\n");
    /* se pinta los puntos sobre el pixmap, en un futuro deberia plantearme
     * cambiar la funcion pixels_changes() para que devuelva un array de
     * GdkPoints, y usar la funcion gdk_draw_points() con ese array en lugar
     * de pintar uno a uno los puntos con gdk_draw_point()
     */
    gc = gdk_gc_new (darea->window);
    gdk_gc_set_rgb_fg_color(gc, &color);
    count = 0;
    for (i = 0; i < image->height; i++)
	for (j = 0; j < image->width; j++)
	    if (changes[i * image->width + j])
	    {
		gdk_draw_point (pixmap, gc, j, i);
		count ++;
	    }
    //printf ("cuenta: %d\n", count);

    g_signal_connect (G_OBJECT(darea), "expose_event", G_CALLBACK(on_histogram_expose_event), pixmap);

    /* se libera el pixmap cuando se cierra la ventana? Imagino que no, lo tengo que eliminar a mano*/
    g_signal_connect (G_OBJECT(window), "destroy", G_CALLBACK(gtk_widget_destroyed), &window);
}
/* **********************************************************************
 * Visualiza la imagen que se le pasa tal cual. Este imagen se corresponde
 * con una imagen de ruido. Los valores que realmente definen este ruido
 * gaussiano (no su representacin grfica) se tienen en el array gauss.
 * ********************************************************************** */
void idp_window_with_gauss (image_t *image, double *gauss)
{
    GtkWidget *window;
    GtkWidget *vbox;		/* contendr al DA y el StatusBar */
    GtkWidget *darea;
    GtkWidget *statusbar;
    GtkWidget *menu = create_menu(image);
    GtkTargetEntry target_table1[] = { 	{TE_TARGET_GAUSS, TE_FLAGS_GAUSS, TE_ID_GAUSS} };
    image_motion_t *im = (image_motion_t*) malloc (sizeof (image_motion_t));
    gauss_t *gg = (gauss_t*) malloc (sizeof (gauss_t));
    
   
    /* se define la ventana y el drawing area que contendr la image */
    darea = gtk_drawing_area_new ();
    gtk_widget_set_size_request (darea, image->width, image->height);
    statusbar = gtk_statusbar_new ();
    window = gtk_window_new (GTK_WINDOW_TOPLEVEL);
    gtk_window_set_title (GTK_WINDOW(window), "Imagen");
    vbox = gtk_vbox_new (FALSE, 8);
    gtk_container_add (GTK_CONTAINER(vbox), darea);
    gtk_container_add (GTK_CONTAINER(vbox), statusbar);
    gtk_container_add (GTK_CONTAINER(window), vbox);
    gtk_container_set_border_width (GTK_CONTAINER(window), 8);
    //gtk_widget_set_events (window, gtk_widget_get_events (window) | GDK_BUTTON_PRESS_MASK);
    gtk_widget_set_events (darea, gtk_widget_get_events (darea) | GDK_POINTER_MOTION_MASK);
    gtk_widget_show_all (window);
    
    /* las seales a atender */
    im->image = image;
    im->statusbar = statusbar;
    gg->image = image;
    gg->gauss = gauss;
    g_signal_connect (G_OBJECT(darea), "expose_event", G_CALLBACK(on_image_expose_event), image);		// redibujado
    g_signal_connect (G_OBJECT(darea), "motion_notify_event", G_CALLBACK(on_image_motion_notify_event), im);	// movimiento de raton
    g_signal_connect (G_OBJECT(window), "destroy", G_CALLBACK(on_gauss_destroyed), gg);				// al destruirse
    g_signal_connect (G_OBJECT(window), "destroy", G_CALLBACK(gtk_widget_destroyed), &window);			// tb al destruirse
    g_signal_connect (G_OBJECT(window), "button_press_event", G_CALLBACK(menu_popup), menu);			// boton raton -> menu

    /* Drag and Drop como destino: NO PERMITIMOS QUE RECIBA NADA */
    /* Drag and Drop como fuente: coger el array que define el ruido (NO COGE LA IMAGEN)  */
    gtk_drag_source_set (window, GDK_BUTTON1_MASK, target_table1, 1, GDK_ACTION_COPY);
    g_signal_connect (G_OBJECT (window), "drag_data_get", G_CALLBACK (on_source_drag_data_get_gauss), gg);
}
/* **********************************************************************
 * Ventana con los grficos de los perfiles.
 * ********************************************************************** */
void idp_window_with_cross_section (int *value, int *value_d, int *value_s_d, int n)
{
    GtkWidget *window, *vbox, *label;
    cross_section_t *cross_section = (cross_section_t*) malloc (sizeof (cross_section_t));
    char str[25];

    vbox = gtk_vbox_new (FALSE, 8);				/* contendr los 3 perfiles */
    gtk_container_set_border_width (GTK_CONTAINER (vbox), 8);
    
    sprintf (str, "Numero Puntos: %d", n);
    label = gtk_label_new (str);
    gtk_box_pack_start (GTK_BOX(vbox), label, FALSE, FALSE, 0);
    
    window = gtk_window_new (GTK_WINDOW_TOPLEVEL);
    gtk_window_set_title (GTK_WINDOW(window), "Perfiles");
    gtk_container_add (GTK_CONTAINER(window), vbox);		/* vbox dentro de la ventana (importante para las proximas llamadas) */

    draw_graph_cross_section (vbox, "Perfil", value, n, 100);				/* se dibuja el perfil */
    draw_graph_cross_section (vbox, "Derivada Perfil", value_d, n, 100);		/* se dibuja la derivada del perfil */
    draw_graph_cross_section (vbox, "Derivada Perfil Suavizado", value_s_d, n, 100);	/* se dibuja la derivada del "perfile suavizado */

    g_signal_connect (G_OBJECT(window), "destroy", G_CALLBACK(gtk_widget_destroyed), &window);
    cross_section -> v = value;
    cross_section -> v_d = value_d;
    cross_section -> v_s_d = value_s_d;
    g_signal_connect (G_OBJECT(window), "destroy", G_CALLBACK(on_cross_section_destroyed), cross_section);
    
    /* !!! he de llamar a una funcin para liberar la memoria asociada a los arrays valuexxx */
    gtk_widget_show_all (window);    
}

/* **********************************************************************
 * Redibuja el histrograma
 * ********************************************************************** */
gboolean on_pixmap_expose_event (GtkWidget *widget, GdkEventExpose *event, GdkPixmap *pixmap)
{
    gdk_draw_drawable(widget->window,
	    widget->style->fg_gc[GTK_WIDGET_STATE(widget)], pixmap,
	    event->area.x,
	    event->area.y,
	    event->area.x,
	    event->area.y,
	    event->area.width,
	    event->area.height);
    
    return TRUE;
}
/* **********************************************************************
 * Redibuja la imagen
 * ********************************************************************** */
gboolean on_image_expose_event (GtkWidget *widget, GdkEventExpose *event, image_t *image)
{
    gdk_draw_indexed_image (
	    widget->window,
	    widget->style->fg_gc[GTK_STATE_NORMAL],//widget->style->fg_gc[GTK_WIDGET_STATE(widget)],
	    0, //event->area.x,
	    0, //event->area.y,
	    image->width, //event->area.x,
	    image->height, //event->area.y,
	    GDK_RGB_DITHER_MAX,
	    image->buf,
	    image->width,
	    image->cmap);

    return TRUE;
}
/* **********************************************************************
 * Redibuja el Look Up Table
 * ********************************************************************** */
gboolean on_lut_expose_event (GtkWidget *widget, GdkEventExpose *event, guchar *colors)
{
    gdk_draw_gray_image (
	    widget->window,
	    widget->style->fg_gc[GTK_STATE_NORMAL],//widget->style->fg_gc[GTK_WIDGET_STATE(widget)],
	    0,
	    0,
	    256,
	    HEIGHT_LUT,
	    GDK_RGB_DITHER_MAX,
	    colors,
	    256);

    return TRUE;
}
/* **********************************************************************
 * Libera la memoria asociada al pixmap
 * ********************************************************************** */
void on_pixmap_destroyed (GtkWidget *widget, GdkPixmap *pixmap)
{
    g_object_unref (pixmap);
}
/* **********************************************************************
 * Libera la imagen cuando se cierra la ventana que la contiene
 * ********************************************************************** */
void on_image_destroyed (GtkWidget *widget, image_t *image)
{
    idp_image_free (image);
}
/* **********************************************************************
 * Se libera la memoria de la imagen (no el look up table) look_up_table
 * cuando se destruye la ventana.
 * ********************************************************************** */
void on_lut_destroyed (GtkWidget *widget, guchar *buf)
{
    free (buf);
}
/* **********************************************************************
 * Se libera la memoria al destruirse la ventana asociada a la imagen
 * de ruido
 * ********************************************************************** */
void on_gauss_destroyed (GtkWidget *widget, gauss_t* gg)
{
    free (gg->image);
    free (gg->gauss);
    free (gg);
}
/* **********************************************************************
 * Func:
 * Desc:
 * ********************************************************************** */
void on_cross_section_destroyed (GtkWidget *widget, cross_section_t *cs)
{
    free (cs->v);
    free (cs->v_d);
    free (cs->v_s_d);
    free (cs);
}
/* **********************************************************************
 * Se han depositado los datos, por tanto, los recogemos. Ser una imagen
 * o un histograma o un ruido gaussiano.
 *  lo que se recibe habr que liberarlo ?
 * ********************************************************************** */
void on_target_drag_data_received (GtkWidget *widget, GdkDragContext *context, guint x, guint y, GtkSelectionData *data, guint info, guint time, gpointer udata)
{
    image_t *image = (image_t*)udata;	/* la imagen asociada a la ventana que recibe los datos */
    image_t *image2, *imaged;
    unsigned int *histogram;
    double *gauss;
    double n;
    
    switch (info)
    {
	case TE_ID_HIST:					/* se recibe un histograma acumulativo */
	    histogram = (unsigned int*) data->data;
	    idp_window_with_image (idp_image_new_histogram_acu (image, histogram));
	    break;
	case TE_ID_IMAGE:					/* se recibe una imagen */
	    image2 = (image_t*) data->data;
	    /* ya tengo las dos imagenes, calculo la imagen diferencia */
	    imaged = idp_image_new_difference (image, image2);
	    idp_window_with_histogram2 (imaged, 1);
	    /* Pregunto por el parametro n */
	    if (!dialog_logarithm_k (&n))
		n = 1.0;	/* valor por defencto */
	    /* Muestro la imagen "image1" con los cambio producidos */
	    idp_window_with_image_and_changes (image, pixels_changes (imaged, n));
	    break;
	case TE_ID_GAUSS:					/* se recibe ruido gaussiano */
	    gauss = (double*) data->data;
	    idp_window_with_image (idp_image_new_add_gauss (image, gauss));
	    g_print ("ey, se recibe el ruido\n");
	    break;
	default: g_print ("DAD: No deberia aparecer este mensaje\n");
    }
}
/* **********************************************************************
 * Se cogen los datos de la ventana origen para enviarlos a donde el
 * usuario los deposite (Se trata de un histograma)
 * ********************************************************************** */
void on_source_drag_data_get_histogram (GtkWidget *widget, GdkDragContext *context, GtkSelectionData *selection_data, guint info, guint time, gpointer data)
{
    gtk_selection_data_set (selection_data, selection_data->target, sizeof(unsigned int) * 8, data, 256 * sizeof (unsigned int));
}
/* **********************************************************************
 * Se cogen los datos de la ventana origen para enviarlos a donde el
 * usuario los deposite (Se trata de una imagen)
 * ********************************************************************** */
void on_source_drag_data_get_image (GtkWidget *widget, GdkDragContext *context, GtkSelectionData *selection_data, guint info, guint time, gpointer data)
{
    gtk_selection_data_set (selection_data, selection_data->target, sizeof(image_t) * 8, data, sizeof (image_t));
}
/* **********************************************************************
 * Se cogen los datos de la ventana origen para enviarlos a donde el
 * usuario los deposite (Se trata de del array de ruido (GAUSSIANO) )
 * ********************************************************************** */
void on_source_drag_data_get_gauss (GtkWidget *widget, GdkDragContext *context, GtkSelectionData *selection_data, guint info, guint time, gpointer data)
{
    gauss_t* gg = (gauss_t*)data;
    gtk_selection_data_set (selection_data, selection_data->target, sizeof(double) * 8, (gpointer)gg->gauss,  gg->image->width * gg->image->height * sizeof (double));
}
/* **********************************************************************
 * Se recoge el movimiento del cursor sobre la imagen y se muestra las
 * coordenadas y el valor del pixel (nivel de gris)
 * ********************************************************************** */
void on_image_motion_notify_event (GtkWidget *widget, GdkEventMotion *event, image_motion_t* im)
{
    int x, y;
    char str[200];
    image_t *image = (image_t*)im->image;

    gtk_widget_get_pointer (widget, &x, &y);
    sprintf (str, "x:%4d y:%4d\tG:%3d", x, y, image->cmap->colors[image->buf[y*image->width+x]]&0xff);
    gtk_statusbar_pop (GTK_STATUSBAR(im->statusbar), 0);
    gtk_statusbar_push (GTK_STATUSBAR(im->statusbar), 0, str);
}
/* **********************************************************************
 * Se recoge el movimiento del cursor sobre el histograma y se muestra
 * el indice a la LUT, el color y el valor del histograma
 * ********************************************************************** */
void on_histogram_motion_notify_event (GtkWidget *widget, GdkEventMotion *event, histogram_motion_t* histogram_motion)
{
    int x, y;
    char str[200];
    image_t *image = (image_t*)histogram_motion->image;
    unsigned int *histogram = histogram_motion->type ? image->hist_abs : image->hist_acu;

    gtk_widget_get_pointer (widget, &x, &y);
    /* porcentaje */
    //sprintf (str, "index:%3d color:%3d value:%.2f", x, image->cmap->colors[x] & 0xff, (float)histogram[x] / (float)(image->width * image->height));
    /* numero de pixeles */
    sprintf (str, "index:%3d color:%3d value:%d", x, image->cmap->colors[x] & 0xff, histogram[x]);
    gtk_statusbar_pop (GTK_STATUSBAR(histogram_motion->statusbar), 0);
    gtk_statusbar_push (GTK_STATUSBAR(histogram_motion->statusbar), 0, str);
}
/* --------------------------------------------------------------------------------------------------------------------------------------------------- */

/* **********************************************************************
 * Crea una look up table para imgenes en escala de grises. Ser 
 * uilizada en la visualizacin de la image en donde se usa la funcin
 * gtk_draw_indexed_image().
 * ********************************************************************** */
GdkRgbCmap *create_gray_lut (void)
{
    guint i;
    guint *colors = (guint*) malloc (sizeof(guint) * 256);
    GdkRgbCmap *cmap;

    for (i = 0; i < 256; i++)
        colors[i] = i + (i<<8) + (i<<16);
    cmap = gdk_rgb_cmap_new (colors, 256);
    free (colors);
    return cmap;
}
/* **********************************************************************
 * Devuelve el histograma absoluto de una image. El nmero de pixeles
 * para cada nivel de gris.
 * ********************************************************************** */
unsigned int *create_absolute_histogram (image_t *image)
{
    unsigned int *histogram;
    int i;

    histogram = (unsigned int*) calloc (256, sizeof (unsigned int));
    for (i = 0; i < image->width * image->height; i++)
	histogram[(image->cmap->colors[image->buf[i]]) & 0xff]++;

    return histogram;
}
/* **********************************************************************
 * Devuelve el histograma acumulativo a partir del histograma absoluto.
 * ********************************************************************** */
unsigned int *create_acumulative_histogram (unsigned int *absolute)
{
    unsigned int *histogram;
    int i;

    histogram = (unsigned int*) calloc (256, sizeof (unsigned int));  
    histogram[0] = absolute[0];
    for (i = 1; i < 256; i++)
	histogram[i] = histogram[i-1] + absolute[i];

    return histogram;
}
/* **********************************************************************
 * Rango de niveles de gris.
 * ********************************************************************** */
void range_of_grays_levels (int *min, int *max, unsigned int *absolute)
{
    int i;

    for (i = 0; i < 256; i++)
	if (absolute[i]) break;
    *min = i;
    for (i = 256; i >= 0; i--)
	if (absolute[i]) break;
    *max = i;
}
/* **********************************************************************
 * Brillo de una image. 
 * ********************************************************************** */
float bright_of_image (image_t *image)
{
    int i;
    float b;
    
    for (i = 0, b = 0; i < 256; i++)
	b += (float)i * (float)image->hist_abs[i];
    b = b / (float)image->hist_acu[255]; 
    return b;
}
/* **********************************************************************
 * Contraste de una image.
 * ********************************************************************** */
float contrast_of_image (image_t *image)
{
    int i;
    float t,c;

    for (i = 0, c = 0; i < 256; i++)
    {
	t = (float)i - image->bright;
	c = c + ((float)image->hist_abs[i] * t * t);
    }
    c = c / (float)image->hist_acu[255];
    return sqrtf (c);
}
/* **********************************************************************
 * Entropia de una image. Lo hace bien?
 * ********************************************************************** */
float entropy_of_image (image_t *image)
{
    int i;
    float t,e;

    for (i = 0, e = 0; i < 256; i++)
    {
	t = (float)image->hist_abs[i] / (float)image->hist_acu[255];
	// La funcion log2 parece no funcionar bien, asi que hago uso
	// de la propiedad de los logaritmos: log2 (x) = Ln(x) / Ln(2)
	if (t != 0.0)
	{
	    t = t * log(t) / LN2;
	    e = e + t;
	}
    }
    e *= -1;
    return e;
}
/* **********************************************************************
 * Develve un array de n numeros con valores entre n1 y n2 que se aproximan
 * a una distribucin normal de media m y desviacin tpica s.
 * La generacin de numeros se hace con el algoritmo de Box-Mller:
 * 	z = (-2 * Ln (u1))^(1/2) * cos (2 * PI * u2)
 * 	donde u1 y u2 son numeros entre 0..1 uniformemente distribuidos
 * 	z es una normal de media 0 y desviacin 1
 * 	x = z*s+m	Normal de media m y desviacion s
 * ********************************************************************** */
double *normal (double m, double s, int n, double n1, double n2)
{
   double *v = (double*) malloc (sizeof (double) * n);
   struct timeval tp;
   int i;
   
   gettimeofday (&tp, NULL);
   srand48((long)tp.tv_usec);	/* semilla */
   
   /* Creamos i numeros aleatorios con distribucin normal de media m y 
    * desviacin tpica s que esten comprendidos en el intervalo
    * [n1,n2]
    */
   for (i = 0; i < n; )
   {
        v[i] = (sqrt(-2 * log(drand48())) * cos(2*PI*(drand48())) * s) + m;
	if ((v[i] >= n1) && (v[i] <= n2))
	    i++;
   }
   return v;
}
/* **********************************************************************
 * Esta funcin es til para operaciones de deteccin de cambios entre 
 * dos imgenes de una misma escena pero adquiridas con una diferencia 
 * temporal. De modo que imaged es la imagen resultante de aplicar la
 * operacin de diferencia entre las dos imgenes de partida. La funcin
 * devolver un array de 0 y 1 indicando de este modo aquellos pxeles
 * que se consideran han experimentado algn cambio. 
 * Se considerar que un pixel de la imagen1 a experimentado un cambio 
 * cuando el nivel de gris asociado a la imagen diferencia est fuera
 * del rango [m - (ns), m + (ns)], con m y s la media y desviacin 
 * tpica de la imagen diferencia y n un valor arbitrario. Este valor
 * har que se considere un nmero menor de cambios cuanto ms grande
 * sea.
 * ********************************************************************** */
char *pixels_changes (image_t* imaged, float n)
{
    int i, size;
    char *change;
    int min, max, cont;

    size = imaged->width * imaged->height;
    change = (char*) malloc (size * sizeof (char));
    min = imaged->bright - (n * imaged->contrast);
    max = imaged->bright + (n * imaged->contrast);
    for (cont = 0, i = 0; i < size; i++)
	if ((imaged->buf[i] < min) || (imaged->buf[i] > max))
	{
	    change[i] = 1;
	    cont++;
	}
	else
	   change[i] = 0;
    return change;
}
/* **********************************************************************
 * Devuelve los puntos que forman el perfil entre dos puntos. Esto es
 * la recta que los une.
 * ********************************************************************** */
GdkPoint *points_to_image_cross_section (int x1, int y1, int x2, int y2, int *n)
{
    int w = abs(x2-x1) + 1;
    int h = abs(y2-y1) + 1;
    float inc_x, inc_y;
    int i,x,y;
    GdkPoint *points;
    
    *n = w > h ? w : h;   
    points = (GdkPoint*) malloc (*n * sizeof (GdkPoint));
    
    if (x2 > x1) inc_x = 1;
    else inc_x = -1;

    if (y2 > y1) inc_y = 1;
    else inc_y = -1;

    if (w > h)
	inc_y *= (float)h / (float)w;
    else
	inc_x *= (float)w / (float)h;

    for (i = 0; i < *n; i++)
    {
	points[i].x = x1 + i * inc_x;
	points[i].y = y1 + i * inc_y;
    }
    return points;
}
/* **********************************************************************
 * Los niveles de gris del perfil. El perfil lo determina los puntos
 * indicados en "points". 
 * ********************************************************************** */
int *value_to_image_cross_section (image_t *image, GdkPoint *points, int n)
{
    int i;
    int *value = (int*) malloc (n * sizeof (int));
    
    for (i = 0; i < n; i++)
	value[i] = image->cmap->colors[image->buf[points[i].y * image->width + points[i].x]] & 0xff;
    return value;
}
/* **********************************************************************
 * Recibe el perfil obtenido de la funcion value_to_image_cross_section()
 * y genera el perfil suavizado. Para ello se agrupan los puntos de tres
 * en tres y se le asiga la media. Tipicamente, el perfile suavizado se
 * pasar a la funcin value_to_image_cross_section_deriv() para calcular
 * la derivada del perfil suavizado que es lo que realmente interesa.
 * ********************************************************************** */
int *value_to_image_cross_section_smoothed (int *value, int n)
{
#define SIZE_GROUP 3
    int i, j;
    int m;
    int *v = (int*) malloc (n * sizeof (int));

    m = n / SIZE_GROUP * SIZE_GROUP;	/* notese que no es lo mismo que "n" */
    for (i = 0; i < m; i++)
    {
	for (v[i] = 0, j = 0; j < SIZE_GROUP; j++)
	    v[i] += value[i+j];
	v[i] = ROUND ((double)v[i] / (double)SIZE_GROUP);
    }
    for (i = m; i < n; i++)	// RESTO DE VALORES QUE NO SE PUEDEN AGRUPAR
	v[i] = value[i];
    return v;
}
/* **********************************************************************
 * Los niveles de gris de la derivada del perfil. La deriva la ajustamos
 * por la diferencia entre los valores de perfil de dos puntos consecutivos.
 * ********************************************************************** */
int *value_to_image_cross_section_deriv (int *value, int n)
{
    int i;
    int *v = (int*) malloc (n * sizeof(int));

    for (i = 0; i < n - 1; i++)
	v[i] = value[i+1] - value[i];
    v[i] = 0;
    return v;
}

/* -------------------------------------------------------------------------------------------------------------------- */

/* **********************************************************************
 * Crea menu emergente
 * ********************************************************************** */
GtkWidget *create_menu (image_t* image)
{
    GtkWidget *menu;
    GtkWidget *item;
    GtkWidget *menu_bits;
    GtkWidget *menu_no_lineal;
    GtkWidget *menu_geometric;
    int i;
    char str[20];
    less_bits_t *lb = (less_bits_t*) malloc (7 * sizeof (less_bits_t));
    
    menu = gtk_menu_new();
    
    item = gtk_menu_item_new_with_label ("Negativo");						/* Item Negativo */
    gtk_menu_append (GTK_MENU(menu), item);
    g_signal_connect (G_OBJECT(item), "activate", G_CALLBACK(menu_negative), image);
    gtk_widget_show (item);

    item = gtk_menu_item_new_with_label ("Ecualizado");						/* Item Ecualizado */
    gtk_menu_append (GTK_MENU(menu), item);
    g_signal_connect (G_OBJECT(item), "activate", G_CALLBACK(menu_equalized), image);
    gtk_widget_show (item);

    item = gtk_menu_item_new_with_label ("Histogramas");					/* Item Histogramas */
    gtk_menu_append (GTK_MENU(menu), item);
    g_signal_connect (G_OBJECT(item), "activate", G_CALLBACK(menu_histogram), image);
    gtk_widget_show (item);

    menu_bits = gtk_menu_new();									/* Items submenu less bits */
    for (i = 7; i > 0; i--)
    {
	sprintf (str, "%d bits", i);
	item = gtk_menu_item_new_with_label (str);
	gtk_menu_append (GTK_MENU(menu_bits), item);
	lb[i].image = image;
	lb[i].id = i;
	g_signal_connect (G_OBJECT(item), "activate", G_CALLBACK(menu_less_bits), &lb[i]);
	gtk_widget_show (item);
    }

    item = gtk_menu_item_new_with_label ("Cuantizacion");					/* Item Less Bits */
    gtk_menu_append (GTK_MENU(menu), item);
    gtk_menu_item_set_submenu (GTK_MENU_ITEM(item), menu_bits);
    gtk_widget_show (item);
    
    item = gtk_menu_item_new_with_label ("Muestreo");						/* Item Less Samples */
    gtk_menu_append (GTK_MENU(menu), item);
    g_signal_connect (G_OBJECT(item), "activate", G_CALLBACK(menu_less_samples), image);
    gtk_widget_show (item);

    item = gtk_menu_item_new_with_label ("Ruido Gaussiano");					/* Item Imagen con ruido */
    gtk_menu_append (GTK_MENU(menu), item);
    g_signal_connect (G_OBJECT(item), "activate", G_CALLBACK(menu_gauss), image);
    gtk_widget_show (item);

    item = gtk_menu_item_new_with_label ("Subimagen");						/* Item subimagen (region de interes) */
    gtk_menu_append (GTK_MENU(menu), item);
    g_signal_connect (G_OBJECT(item), "activate", G_CALLBACK(menu_subimage), image);
    gtk_widget_show (item);

    item = gtk_menu_item_new_with_label ("Brillo y Contaste");					/* Item modificar brillo y contraste */
    gtk_menu_append (GTK_MENU(menu), item);
    g_signal_connect (G_OBJECT(item), "activate", G_CALLBACK(menu_bright_contrast), image);
    gtk_widget_show (item);
    
    item = gtk_menu_item_new_with_label ("Ajuste Lineal");					/* Item Ajuste Lineal */
    gtk_menu_append (GTK_MENU(menu), item);
    g_signal_connect (G_OBJECT(item), "activate", G_CALLBACK(menu_lineal), image);
    gtk_widget_show (item);

    item = gtk_menu_item_new_with_label ("Perfil");						/* Item Perfil */
    gtk_menu_append (GTK_MENU(menu), item);
    g_signal_connect (G_OBJECT(item), "activate", G_CALLBACK(menu_cross_section), image);
    gtk_widget_show (item);

    menu_no_lineal = gtk_menu_new ();								/* Items submenu no lineal */
    item = gtk_menu_item_new_with_label ("Logaritmo");
    gtk_menu_append (GTK_MENU (menu_no_lineal), item);
    g_signal_connect (G_OBJECT(item), "activate", G_CALLBACK(menu_logarithm), image);
    gtk_widget_show (item);
    
    item = gtk_menu_item_new_with_label ("Logaritmo K");
    gtk_menu_append (GTK_MENU (menu_no_lineal), item);
    g_signal_connect (G_OBJECT(item), "activate", G_CALLBACK(menu_logarithm_k), image);
    gtk_widget_show (item);
    
    item = gtk_menu_item_new_with_label ("Logaritmo Histograma");
    gtk_menu_append (GTK_MENU (menu_no_lineal), item);
    g_signal_connect (G_OBJECT(item), "activate", G_CALLBACK(menu_logarithm_histogram_k), image);
    gtk_widget_show (item);
    
    item = gtk_menu_item_new_with_label ("Exponencial");
    gtk_menu_append (GTK_MENU (menu_no_lineal), item);
    g_signal_connect (G_OBJECT(item), "activate", G_CALLBACK(menu_exponential), image);
    gtk_widget_show (item);
    
    item = gtk_menu_item_new_with_label ("Exponencial K");
    gtk_menu_append (GTK_MENU (menu_no_lineal), item);
    g_signal_connect (G_OBJECT(item), "activate", G_CALLBACK(menu_exponential_k), image);
    gtk_widget_show (item);
    
    item = gtk_menu_item_new_with_label ("Exponencial Histograma");
    gtk_menu_append (GTK_MENU (menu_no_lineal), item);
    g_signal_connect (G_OBJECT(item), "activate", G_CALLBACK(menu_exponential_histogram_k), image);
    gtk_widget_show (item);

    item = gtk_menu_item_new_with_label ("No Lineal");						/* Item no lineal */
    gtk_menu_append (GTK_MENU(menu), item);
    gtk_menu_item_set_submenu (GTK_MENU_ITEM(item), menu_no_lineal);
    gtk_widget_show (item);

    /* menu transformaciones geomtricas */

    menu_geometric = gtk_menu_new ();							/* Items submenu no lineal */
    item = gtk_menu_item_new_with_label ("Espejo Vertical");
    gtk_menu_append (GTK_MENU (menu_geometric), item);
    g_signal_connect (G_OBJECT(item), "activate", G_CALLBACK(menu_vertical_mirror), image);
    gtk_widget_show (item);

    item = gtk_menu_item_new_with_label ("Espejo Horizontal");
    gtk_menu_append (GTK_MENU (menu_geometric), item);
    g_signal_connect (G_OBJECT(item), "activate", G_CALLBACK(menu_horizontal_mirror), image);
    gtk_widget_show (item);
    
    item = gtk_menu_item_new_with_label ("Rotar 90");
    gtk_menu_append (GTK_MENU (menu_geometric), item);
    g_signal_connect (G_OBJECT(item), "activate", G_CALLBACK(menu_rotation_90), image);
    gtk_widget_show (item);
    
    item = gtk_menu_item_new_with_label ("Rotar ...");
    gtk_menu_append (GTK_MENU (menu_geometric), item);
    g_signal_connect (G_OBJECT(item), "activate", G_CALLBACK(menu_rotation), image);
    gtk_widget_show (item);
    
    item = gtk_menu_item_new_with_label ("Zoom ...");
    gtk_menu_append (GTK_MENU (menu_geometric), item);
    g_signal_connect (G_OBJECT(item), "activate", G_CALLBACK(menu_resize), image);
    gtk_widget_show (item);

    item = gtk_menu_item_new_with_label ("T Geometricas");					/* Item T. Geomtricas */
    gtk_menu_append (GTK_MENU(menu), item);
    gtk_menu_item_set_submenu (GTK_MENU_ITEM(item), menu_geometric);
    gtk_widget_show (item);
    
    
    return menu;
}
/* **********************************************************************
 * Visuailza menu emergente
 * ********************************************************************** */
gint menu_popup (GtkWidget *widget, GdkEventButton *event_button, gpointer data)
{
    if (event_button->button == 3)
	gtk_menu_popup (GTK_MENU(data), NULL, NULL, NULL, NULL, event_button->button, event_button->time);

    /* retorno falso para que el evento no desaparezca, de esta manera puedo
     * usar el Drag And Drop cuano pulso el boton del medio */
    return FALSE;
}
/* **********************************************************************
 * Interfaz para la funcin del negativo
 * ********************************************************************** */
gint menu_negative (GtkWidget *widget, gpointer data)
{
    image_t *image = (image_t*)data;
    idp_window_with_image (idp_image_new_negative (image));
    return TRUE;
}
/* **********************************************************************
 * Interfaz para la ecualizacin del histograma de una imagen
 * ********************************************************************** */
gint menu_equalized (GtkWidget *widget, gpointer data)
{
    image_t *image = (image_t*)data;
    idp_window_with_image (idp_image_new_equalized (image));
    return TRUE;
}
/* **********************************************************************
 * Interfaz para los histogramas
 * ********************************************************************** */
gint menu_histogram (GtkWidget *widget, gpointer data)
{
    image_t *image = (image_t*)data;
    idp_window_with_histogram2 (image, 1);
    idp_window_with_histogram2 (image, 0);
    return TRUE;
}
/* **********************************************************************
 * Interfaz para la simulacin de la cuantizacin
 * ********************************************************************** */
gint menu_less_bits (GtkWidget *widget, gpointer data)
{
    less_bits_t *lb = (less_bits_t*)data;
    idp_window_with_image  (idp_image_new_less_bits (lb->image,lb->id));
}
/* **********************************************************************
 * Interfaz para la simulacin del muestreo
 * ********************************************************************** */
gint menu_less_samples (GtkWidget *widget, gpointer data)
{
    image_t *image = (image_t*)data;
    int x = image->width, y = image->height;

    if (dialog_less_samples (&x, &y))
	idp_window_with_image  (idp_image_new_less_samples (image, x, y));
}
/* **********************************************************************
 * Interfaz para la creacin de una imagen con ruido gaussiano. Lo 
 * nico que nos interesa de la imagen que se le pasa es su alto
 * y ancho.
 * ********************************************************************** */
gint menu_gauss (GtkWidget *widget, gpointer data)
{
    image_t *image = (image_t*)data;
    int width, height;		/* ancho y alto de la imagen */
    double m, s, n1, n2;	/* media, desviacion tipia, rango [n1,n2] */
    double *gauss;
    image_t *new;

    width = image->width;	/* lo normal es que se mantengan estas dimensiones */
    height = image->height;
    if (dialog_gauss (&width, &height, &m, &s, &n1, &n2))
    {
	new = idp_image_new_gauss (width, height, m, s, n1, n2, &gauss);
	idp_window_with_gauss (new, gauss);
    }
}
/* **********************************************************************
 * Interfaz para la obtencin de una subimagen o regin de inters
 * ********************************************************************** */
gint menu_subimage (GtkWidget *widget, gpointer data)
{
    image_t *image = (image_t*)data;
    int x = 0 , y = 0, width = image->width, height = image->height;	/* Lmetes de la iamgen */

    if (dialog_subimage (&x, &y, &width, &height))
	idp_window_with_image  (idp_image_new_subimage (image, x, y, width, height));
}
/* **********************************************************************
 * Interfaz para la creacin de una nueva imagen con nuevos valores de
 * brillo y contraste
 * ********************************************************************** */
gint menu_bright_contrast (GtkWidget *widget, gpointer data)
{
    image_t *image = (image_t*)data;
    float bright = image->bright, contrast = image->contrast;

    if (dialog_bright_contrast (&bright, &contrast))
	idp_window_with_image  (idp_image_new_bright_contrast (image, bright, contrast));
}
/* **********************************************************************
 * Interfaz para el ajuste lineal
 * ********************************************************************** */
gint menu_lineal (GtkWidget *widget, gpointer data)
{
    image_t *image = (image_t*)data;
    int *points = (int*) malloc (256 * sizeof (int));
    int i;

    for (i = 1; i < 255; i++)
	points[i] = -1;
    points[0] = 0;		/* tanto el primero como el ltimo deben estar definidos */
    points[255] = 255; 
    if (dialog_lineal (points))
	idp_window_with_image  (idp_image_new_lineal (image, points));
    free (points);
}
/* **********************************************************************
 * Interfaz para la obtencin de una subimagen o regin de inters
 * ********************************************************************** */
gint menu_cross_section (GtkWidget *widget, gpointer data)
{
    image_t *image = (image_t*)data;
    int x1 = 0 , y1 = 0, x2 = image->width, y2 = image->height;		/* Lmetes de la iamgen */
    int n;		/* numero de puntos del perfil */
    GdkPoint *points;	/* puntos del perfil */
    int *value;		/* perfil */
    int *value_s;	/* perfil suavizado (smoothed) */
    int *value_d;	/* derivada del perfil */
    int *value_s_d;	/* derivada del "perfil suavizado" */

    if (dialog_cross_section (&x1, &y1, &x2, &y2))
    {
	points = points_to_image_cross_section (x1, y1, x2, y2, &n);
	value = value_to_image_cross_section (image, points, n);	/* perfil */
	value_s = value_to_image_cross_section_smoothed (value, n);
	value_d = value_to_image_cross_section_deriv (value, n);	/* derivada del perfil */
	value_s_d = value_to_image_cross_section_deriv (value_s, n);	/* derivada del perfil suavizado */
	/* se llama a dibuajar los 3 perfiles */
	idp_window_with_cross_section (value, value_d, value_s_d, n);
	/* Solo se libera points y value_s ya que el resto de arrays es necesario para
	 * redibujar el pixmap cuando este cambie de tamao.
	 * Estos arrays se liberaran al cerrar la ventana que contenga a los perfiles */
	free (points);
	free (value_s);
    }
}
/* **********************************************************************
 * Interfaz para ajuste logartmico
 * ********************************************************************** */
gint menu_logarithm (GtkWidget *widget, gpointer data)
{
    image_t* image = (image_t*)data;
    
    idp_window_with_image (idp_image_new_logarithm (image));
}
/* **********************************************************************
 * Interfaz para ajuste logartmico con parmetro k
 * ********************************************************************** */
gint menu_logarithm_k (GtkWidget *widget, gpointer data)
{
    image_t* image = (image_t*)data;
    double k;
    
    if (dialog_logarithm_k (&k))
	idp_window_with_image (idp_image_new_logarithm_k (image, k));
}
/* **********************************************************************
 * Interfaz para ajuste logartmico con parmetro k sobre el histograma
 * ********************************************************************** */
gint menu_logarithm_histogram_k (GtkWidget *widget, gpointer data)
{
    image_t* image = (image_t*)data;
    double k;
    
    if (dialog_logarithm_k (&k))
	idp_window_with_image (idp_image_new_logarithm_histogram_k (image, k));
}
/* **********************************************************************
 * Interfaz para ajuste exponencial
 * ********************************************************************** */
gint menu_exponential (GtkWidget *widget, gpointer data)
{
    image_t* image = (image_t*)data;
    
    idp_window_with_image (idp_image_new_exponential (image));
}

/* **********************************************************************
 * Interfaz para ajuste exponencial con parmetro k sobre el histograma
 * ********************************************************************** */
gint menu_exponential_k (GtkWidget *widget, gpointer data)
{
    image_t* image = (image_t*)data;
    double k;
    
    if (dialog_logarithm_k (&k))	/* el dialogo del logaritmo me sirve para la de la exponencial */
	idp_window_with_image (idp_image_new_exponential_k (image, k));
}
/* **********************************************************************
 * Interfaz para ajuste exponencial con parmetro k sobre el histograma
 * ********************************************************************** */
gint menu_exponential_histogram_k (GtkWidget *widget, gpointer data)
{
    image_t* image = (image_t*)data;
    double k;
    
    if (dialog_logarithm_k (&k))	/* el dialogo me sirve */
	idp_window_with_image (idp_image_new_exponential_histogram_k (image, k));
}

/* XXXXXXXXXXXXXXX MENU PARA TRANSFORMACIONES GEOMETRICAS XXXXXXXXXXXXXXXXXXXXXX */

/* **********************************************************************
 * Interfaz para el espejo de la imagen en vertical
 * ********************************************************************** */
gint menu_vertical_mirror (GtkWidget *widget, gpointer data)
{
    image_t* image = (image_t*)data;
    idp_window_with_image (idp_image_new_vertical_mirror (image));
}

/* **********************************************************************
 * Interfaz para el espejo de la imagen en horizontal
 * ********************************************************************** */
gint menu_horizontal_mirror (GtkWidget *widget, gpointer data)
{
    image_t* image = (image_t*)data;
    idp_window_with_image (idp_image_new_horizontal_mirror (image));
}

/* **********************************************************************
 * Interfaz para la rotacin de 90 grados
 * ********************************************************************** */
gint menu_rotation_90 (GtkWidget *widget, gpointer data)
{
    image_t* image = (image_t*)data;
    idp_window_with_image (idp_image_new_rotation_90 (image));
}

/* **********************************************************************
 * Interfaz para la rotacin
 * ********************************************************************** */
gint menu_rotation (GtkWidget *widget, gpointer data)
{
    int degree;	/* ngulo */
    int type;	/* tipo de interpolacin */
    int direct;	/* transformacin directa o inversa */
    image_t* image = (image_t*)data;
    if (dialog_rotation (&degree, &type, &direct))
	idp_window_with_image (idp_image_new_rotation (image, degree, type, direct));
}

/* **********************************************************************
 * Interfaz para el escalado 
 * ********************************************************************** */
gint menu_resize (GtkWidget *widget, gpointer data)
{
    int percent_x, percent_y, type;
    image_t* image = (image_t*)data;
    if (dialog_resize (&percent_x, &percent_y, &type))
	idp_window_with_image (idp_image_new_resize (image, percent_x, percent_y, type));
}

/* ------------------------------------------------------------------------------------------------------------- */

/* **********************************************************************
 * Muestra un dilogo con dos entradas nmericas para seleccionar las
 * agrupaciones en x y en y para simular un muestreo. Inicialmente
 * x e y debern contener las dimensiones e la imagen.
 * ********************************************************************** */
unsigned char dialog_less_samples (int *x, int *y)
{
    GtkWidget *dialog, *frame, *spin_x, *spin_y;
    gint response;
    
    dialog = gtk_dialog_new_with_buttons ("Simulacion de Muestreo", NULL, GTK_DIALOG_MODAL| GTK_DIALOG_DESTROY_WITH_PARENT,
	    GTK_STOCK_OK, GTK_RESPONSE_OK, GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL, NULL);

    frame = frame_two_spin_buttons ("Agrupe", 1, *x, 1, *y, &spin_x, &spin_y);
    gtk_widget_show_all (frame);
    gtk_box_pack_start (GTK_BOX(GTK_DIALOG(dialog)->vbox), frame, FALSE, FALSE, 0);
    response = gtk_dialog_run (GTK_DIALOG(dialog));
    if (response == GTK_RESPONSE_OK)
    {
        *x = gtk_spin_button_get_value_as_int (GTK_SPIN_BUTTON(spin_x));
	*y = gtk_spin_button_get_value_as_int (GTK_SPIN_BUTTON(spin_y));
	response = 1;
    }
    else
	response = 0;
    gtk_widget_destroy (dialog);
    return response;
}
/* **********************************************************************
 * Muestra un dilog donde seleccionar los valores necesarios para
 * generar una imagen con ruido segn una distribucin gaussiana (normal)
 * Inicialmente width y height contendrn las dimensiones de la iamgen.
 * ********************************************************************** */
unsigned char dialog_gauss (int* width, int* height, double* m, double* s, double* n1, double* n2)
{
    GtkWidget *dialog, *frame, *spin_w, *spin_h, *spin_m, *spin_s, *spin_n1, *spin_n2;
    gint response;
    
    *m = 0; *s = 1; *n1 = -5; *n2 = 5;	/* Valores por defecto */ 

    dialog = gtk_dialog_new_with_buttons ("Ruido Gaussiano", NULL, GTK_DIALOG_MODAL| GTK_DIALOG_DESTROY_WITH_PARENT,
	    GTK_STOCK_OK, GTK_RESPONSE_OK, GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL, NULL);

    frame = frame_gauss ("Ruido Gaussiano", *width, *height, &spin_w, &spin_h, &spin_m, &spin_s, &spin_n1, &spin_n2); 
    gtk_widget_show_all (frame);
    gtk_box_pack_start (GTK_BOX(GTK_DIALOG(dialog)->vbox), frame, FALSE, FALSE, 0);
    response = gtk_dialog_run (GTK_DIALOG(dialog));
    if (response == GTK_RESPONSE_OK)
    {
        *width = gtk_spin_button_get_value_as_int (GTK_SPIN_BUTTON(spin_w));
	*height = gtk_spin_button_get_value_as_int (GTK_SPIN_BUTTON(spin_h));
        *m = gtk_spin_button_get_value (GTK_SPIN_BUTTON(spin_m));
	*s = gtk_spin_button_get_value (GTK_SPIN_BUTTON(spin_s));
        *n1 = gtk_spin_button_get_value (GTK_SPIN_BUTTON(spin_n1));
	*n2 = gtk_spin_button_get_value (GTK_SPIN_BUTTON(spin_n2));
	response = 1;
    }
    else
	response = 0;
    gtk_widget_destroy (dialog);
    return response;
}
/* **********************************************************************
 * Muestra un dilog donde seleccionar los dos puntos que determinan
 * la regin de la subimagen. Inicialmente width y height contendrn
 * las dimensiones de la imagen
 * ********************************************************************** */
unsigned char dialog_subimage (int* x, int* y, int* width, int* height)
{
    GtkWidget *dialog, *frame1, *frame2, *spin_x1, *spin_y1, *spin_x2, *spin_y2;
    GtkWidget *hbox;
    gint response;
    
    dialog = gtk_dialog_new_with_buttons ("Selecione region interes", NULL, GTK_DIALOG_MODAL| GTK_DIALOG_DESTROY_WITH_PARENT,
	    GTK_STOCK_OK, GTK_RESPONSE_OK, GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL, NULL);

    frame1 = frame_two_spin_buttons ("Punto 1", 0, *width, 0, *height, &spin_x1, &spin_y1);
    frame2 = frame_two_spin_buttons ("Punto 2", 0, *width, 0, *height, &spin_x2, &spin_y2);
    hbox = gtk_hbox_new (FALSE, 8);
    gtk_box_pack_start (GTK_BOX(hbox), frame1, FALSE, FALSE, 0);
    gtk_box_pack_start (GTK_BOX(hbox), frame2, FALSE, FALSE, 0);
    gtk_box_pack_start (GTK_BOX(GTK_DIALOG(dialog)->vbox), hbox, FALSE, FALSE, 0);
    gtk_widget_show_all (hbox);
    response = gtk_dialog_run (GTK_DIALOG(dialog));
    if (response == GTK_RESPONSE_OK)
    {
        *x = gtk_spin_button_get_value_as_int (GTK_SPIN_BUTTON(spin_x1));
	*y = gtk_spin_button_get_value_as_int (GTK_SPIN_BUTTON(spin_y1));
        *width = gtk_spin_button_get_value_as_int (GTK_SPIN_BUTTON(spin_x2));
	*height =gtk_spin_button_get_value_as_int (GTK_SPIN_BUTTON(spin_y2));
	*width -= *x;
	*height -= *y;
	response = 1;
    }
    else
	response = 0;
    gtk_widget_destroy (dialog);
    return response;
}
/* **********************************************************************
 * Muestra un dilog donde seleccionar los nuevos valores de brillo y
 * contraste de la imagen. Inicialmente bright y contrast contendrn
 * los valores de brillo y contraste de la imagen.
 * ********************************************************************** */
unsigned char dialog_bright_contrast (float* bright, float* contrast)
{
    GtkWidget *dialog, *frame, *hscale_b, *vbox, *hscale_c;
    gint response;
    
    dialog = gtk_dialog_new_with_buttons ("Brillo y Contraste", NULL, GTK_DIALOG_MODAL| GTK_DIALOG_DESTROY_WITH_PARENT,
	    GTK_STOCK_OK, GTK_RESPONSE_OK, GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL, NULL);

    /* frame <-- vbox <-- barras */
    vbox = gtk_vbox_new (FALSE, 8);
    
    /* Barra brillo */
    frame = gtk_frame_new ("Brillo");
    gtk_container_set_border_width (GTK_CONTAINER(frame), 8);
    hscale_b = gtk_hscale_new_with_range (0, 255, 1);
    gtk_widget_set_usize (GTK_WIDGET (hscale_b), 256, 30);
    gtk_range_set_value (GTK_RANGE(hscale_b), *bright);
    gtk_scale_set_digits (GTK_SCALE(hscale_b), 0);
    gtk_scale_set_value_pos (GTK_SCALE (hscale_b), GTK_POS_TOP);
    gtk_scale_set_draw_value (GTK_SCALE (hscale_b), TRUE);
    gtk_container_add (GTK_CONTAINER(frame), hscale_b);
    gtk_box_pack_start (GTK_BOX (vbox), frame, TRUE, TRUE, 0);

    /* Barra Contraste */
    frame = gtk_frame_new ("Contraste");
    gtk_container_set_border_width (GTK_CONTAINER(frame), 8);
    hscale_c = gtk_hscale_new_with_range (0, 255, 1);
    gtk_widget_set_usize (GTK_WIDGET (hscale_c), 256, 30);
    gtk_range_set_value (GTK_RANGE(hscale_c), *contrast);
    gtk_scale_set_digits (GTK_SCALE(hscale_c), 0);
    gtk_scale_set_value_pos (GTK_SCALE (hscale_c), GTK_POS_TOP);
    gtk_scale_set_draw_value (GTK_SCALE (hscale_c), TRUE);
    gtk_container_add (GTK_CONTAINER(frame), hscale_c);
    gtk_box_pack_start (GTK_BOX (vbox), frame, TRUE, TRUE, 0);
    
    gtk_widget_show_all (vbox);
    gtk_box_pack_start (GTK_BOX(GTK_DIALOG(dialog)->vbox), vbox, FALSE, FALSE, 0);
    response = gtk_dialog_run (GTK_DIALOG(dialog));
    if (response == GTK_RESPONSE_OK)
    {
        *bright = gtk_range_get_value (GTK_RANGE(hscale_b));
	*contrast = gtk_range_get_value (GTK_RANGE(hscale_c));
	response = 1;
    }
    else
	response = 0;
    gtk_widget_destroy (dialog);
    return response;
}
/* **********************************************************************
 * Muestra un dilog donde seleccionar los puntos que se usarn para
 * realizar un ajuste lineal
 * ********************************************************************** */
unsigned char dialog_lineal (int *points)
{
    GtkWidget *dialog, *frame, *spin_x, *spin_y;
    GtkWidget *hbox;
    GtkWidget *darea;
    GdkPixmap *pixmap;

    gint response;
    int x, y;
    lineal_t *lineal_v = (lineal_t*) malloc (sizeof (lineal_t));
   
    dialog = gtk_dialog_new_with_buttons ("Selecciones los tramos", NULL, GTK_DIALOG_MODAL| GTK_DIALOG_DESTROY_WITH_PARENT,
	    GTK_STOCK_OK, GTK_RESPONSE_OK, GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL, NULL);

    frame = frame_two_spin_buttons ("Punto", 0, 255 , -1, 255, &spin_x, &spin_y);
    darea = gtk_drawing_area_new ();
    gtk_widget_set_size_request (darea, 256, 256);
    
    hbox = gtk_hbox_new (FALSE, 8);
    gtk_box_pack_start (GTK_BOX(hbox), frame, TRUE, TRUE, 0);
    gtk_box_pack_start (GTK_BOX(hbox), darea, TRUE, TRUE, 0);
    frame = gtk_frame_new ("Ajuste Lineal Tramos");
    gtk_container_set_border_width (GTK_CONTAINER(frame), 8);
    gtk_container_add (GTK_CONTAINER(frame), hbox);
    gtk_box_pack_start (GTK_BOX(GTK_DIALOG(dialog)->vbox), frame, FALSE, FALSE, 0);
    gtk_widget_show_all (dialog);    
    
    pixmap = gdk_pixmap_new (darea->window, 255, 255, -1);	/* pixmap para los tramos ajuste lineal */
    //gdk_draw_rectangle (pixmap, darea->style->white_gc, TRUE, 0, 0, 255, 255);
    draw_lineal (darea, pixmap, points, 2);

    g_signal_connect (G_OBJECT(darea), "expose_event", G_CALLBACK(on_lineal_expose_event), pixmap);
    g_signal_connect (G_OBJECT(darea), "destroy", G_CALLBACK(on_lineal_destroyed), pixmap);

    // capturo las seales de modificacin  de los spin para actuar en consecuencia
    // Tendr que pasar tanto el array points como el futuro DA
    // Tambin tendr que pasar el spint button complementario
    // Esto lo contengo en una estructura de tipo "lineal_t"
    lineal_v->points = points;
    lineal_v->spin_y = spin_y;
    lineal_v->spin_x = spin_x;
    lineal_v->n = 2;
    lineal_v->pixmap = pixmap;
    lineal_v->darea = darea;
    g_signal_connect (GTK_SPIN_BUTTON (spin_x), "value_changed", G_CALLBACK (on_spinx_lineal_value_changed), lineal_v);
    g_signal_connect (GTK_SPIN_BUTTON (spin_y), "value_changed", G_CALLBACK (on_spiny_lineal_value_changed), lineal_v);
    
    response = gtk_dialog_run (GTK_DIALOG(dialog));
    if (response == GTK_RESPONSE_OK)
	response = 1;
    else
	response = 0;
    gtk_widget_destroy (dialog);
    return response;
}
/* **********************************************************************
 * Muestra un dilog donde seleccionar los dos puntos que determinan
 * la regin de la subimagen. Inicialmente width y height contendrn
 * las dimensiones de la imagen
 * ********************************************************************** */
unsigned char dialog_cross_section (int* x1, int* y1, int* x2, int* y2)
{
    GtkWidget *dialog, *frame1, *frame2, *spin_x1, *spin_y1, *spin_x2, *spin_y2;
    GtkWidget *hbox;
    gint response;
    
    dialog = gtk_dialog_new_with_buttons ("Selecione dos puntos", NULL, GTK_DIALOG_MODAL| GTK_DIALOG_DESTROY_WITH_PARENT,
	    GTK_STOCK_OK, GTK_RESPONSE_OK, GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL, NULL);

    frame1 = frame_two_spin_buttons ("Punto 1", 0, *x2, 0, *y2, &spin_x1, &spin_y1);
    frame2 = frame_two_spin_buttons ("Punto 2", 0, *x2, 0, *y2, &spin_x2, &spin_y2);
    hbox = gtk_hbox_new (FALSE, 8);
    gtk_box_pack_start (GTK_BOX(hbox), frame1, FALSE, FALSE, 0);
    gtk_box_pack_start (GTK_BOX(hbox), frame2, FALSE, FALSE, 0);
    gtk_box_pack_start (GTK_BOX(GTK_DIALOG(dialog)->vbox), hbox, FALSE, FALSE, 0);
    gtk_widget_show_all (hbox);
    response = gtk_dialog_run (GTK_DIALOG(dialog));
    if (response == GTK_RESPONSE_OK)
    {
        *x1 = gtk_spin_button_get_value_as_int (GTK_SPIN_BUTTON(spin_x1));
	*y1 = gtk_spin_button_get_value_as_int (GTK_SPIN_BUTTON(spin_y1));
        *x2 = gtk_spin_button_get_value_as_int (GTK_SPIN_BUTTON(spin_x2));
	*y2 =gtk_spin_button_get_value_as_int (GTK_SPIN_BUTTON(spin_y2));
    }
    else
	response = 0;
    gtk_widget_destroy (dialog);
    return response;
}
/* **********************************************************************
 * Muestra un dilogo donde elegir el parmetro k del logaritmo
 * ********************************************************************** */
unsigned char dialog_logarithm_k (double *k)
{
#define MIN_K 0.1
#define MAX_K 10.0
    GtkWidget *dialog, *frame, *spin_k, *label, *hbox;
    gint response;
    
    dialog = gtk_dialog_new_with_buttons ("Seleccione el parametro", NULL, GTK_DIALOG_MODAL| GTK_DIALOG_DESTROY_WITH_PARENT,
	    GTK_STOCK_OK, GTK_RESPONSE_OK, GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL, NULL);

    hbox = gtk_hbox_new (FALSE, 8);
    label = gtk_label_new_with_mnemonic ("_k");
    gtk_box_pack_start (GTK_BOX (hbox), label, FALSE, FALSE, 0);
    spin_k = gtk_spin_button_new_with_range (MIN_K, MAX_K, 0.1);
    gtk_spin_button_set_value ((GtkSpinButton*)spin_k, log(256));	/* por defecto se corresponde a un logaritmo NORMAL (sin parametro k) */
    gtk_box_pack_start (GTK_BOX (hbox), spin_k, FALSE, FALSE, 0);
    gtk_label_set_mnemonic_widget (GTK_LABEL (label), spin_k);

    frame = gtk_frame_new ("Parametro k");
    gtk_container_set_border_width (GTK_CONTAINER(frame), 8);
    gtk_container_add (GTK_CONTAINER(frame), hbox);
    
    gtk_widget_show_all (frame);
    gtk_box_pack_start (GTK_BOX(GTK_DIALOG(dialog)->vbox), frame, FALSE, FALSE, 0);
    response = gtk_dialog_run (GTK_DIALOG(dialog));
    if (response == GTK_RESPONSE_OK)
    {
        *k = gtk_spin_button_get_value (GTK_SPIN_BUTTON(spin_k));
	response = 1;
    }
    else
	response = 0;
    gtk_widget_destroy (dialog);
    return response;
}

/* *************************************************************************
 * Muestra un dilogo donde el ngulo de rotacin y el tipo de interpolacin
 * ************************************************************************* */
unsigned char dialog_rotation (int *degree, int *type, int *direct)
{
#define MIN_DEGREE -359
#define MAX_DEGREE  359
    GtkWidget *dialog, *frame, *spin_d, *spin_t, *spin_r, *label, *hbox;
    gint response;

    dialog = gtk_dialog_new_with_buttons ("Rotacion", NULL, GTK_DIALOG_MODAL| GTK_DIALOG_DESTROY_WITH_PARENT,
	    GTK_STOCK_OK, GTK_RESPONSE_OK, GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL, NULL);

    hbox = gtk_hbox_new (FALSE, 8);
    
    label = gtk_label_new_with_mnemonic ("_Angulo");
    gtk_box_pack_start (GTK_BOX (hbox), label, FALSE, FALSE, 0);
    spin_d = gtk_spin_button_new_with_range (MIN_DEGREE, MAX_DEGREE, 1.0);
    gtk_spin_button_set_value ((GtkSpinButton*)spin_d, 0);
    gtk_box_pack_start (GTK_BOX (hbox), spin_d, FALSE, FALSE, 0);
    gtk_label_set_mnemonic_widget (GTK_LABEL (label), spin_d);

    label = gtk_label_new_with_mnemonic ("_Interpolacion");
    gtk_box_pack_start (GTK_BOX (hbox), label, FALSE, FALSE, 0);
    spin_t = gtk_spin_button_new_with_range (0, 1, 1.0);
    gtk_spin_button_set_value ((GtkSpinButton*)spin_t, 0);
    gtk_box_pack_start (GTK_BOX (hbox), spin_t, FALSE, FALSE, 0);
    gtk_label_set_mnemonic_widget (GTK_LABEL (label), spin_t);
    
    label = gtk_label_new_with_mnemonic ("_Transformacion (Directa)");
    gtk_box_pack_start (GTK_BOX (hbox), label, FALSE, FALSE, 0);
    spin_r = gtk_spin_button_new_with_range (0, 1, 1.0);
    gtk_spin_button_set_value ((GtkSpinButton*)spin_r, 0);
    gtk_box_pack_start (GTK_BOX (hbox), spin_r, FALSE, FALSE, 0);
    gtk_label_set_mnemonic_widget (GTK_LABEL (label), spin_r);
    
    frame = gtk_frame_new ("Interpolacion -- 1: bilineal -- 0 vec. max cercano ");
    gtk_container_set_border_width (GTK_CONTAINER(frame), 8);
    gtk_container_add (GTK_CONTAINER(frame), hbox);
    
    gtk_widget_show_all (frame);
    gtk_box_pack_start (GTK_BOX(GTK_DIALOG(dialog)->vbox), frame, FALSE, FALSE, 0);
    response = gtk_dialog_run (GTK_DIALOG(dialog));
    if (response == GTK_RESPONSE_OK)
    {
        *degree = gtk_spin_button_get_value (GTK_SPIN_BUTTON(spin_d));
	*type = gtk_spin_button_get_value (GTK_SPIN_BUTTON(spin_t));
	*direct = gtk_spin_button_get_value (GTK_SPIN_BUTTON(spin_r));
	response = 1;
    }
    else
	response = 0;
    gtk_widget_destroy (dialog);
    return response;
}

/* *************************************************************************
 * Muestra un dilogo donde seleccionar el porcentaje del redimensionado
 * y el tipo de interpolacin
 * ************************************************************************* */
unsigned char dialog_resize (int *percent_x, int *percent_y, int *type)
{
#define MIN_PERCENT -99
#define MAX_PERCENT  400
    GtkWidget *dialog, *frame, *spin_x, *spin_y, *spin_t, *label, *hbox;
    gint response;

    dialog = gtk_dialog_new_with_buttons ("Rotacion", NULL, GTK_DIALOG_MODAL| GTK_DIALOG_DESTROY_WITH_PARENT,
	    GTK_STOCK_OK, GTK_RESPONSE_OK, GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL, NULL);

    hbox = gtk_hbox_new (FALSE, 8);
    
    label = gtk_label_new_with_mnemonic ("Porcentaje _X");
    gtk_box_pack_start (GTK_BOX (hbox), label, FALSE, FALSE, 0);
    spin_x = gtk_spin_button_new_with_range (MIN_DEGREE, MAX_DEGREE, 1.0);
    gtk_spin_button_set_value ((GtkSpinButton*)spin_x, 0);
    gtk_box_pack_start (GTK_BOX (hbox), spin_x, FALSE, FALSE, 0);
    gtk_label_set_mnemonic_widget (GTK_LABEL (label), spin_x);
    
    label = gtk_label_new_with_mnemonic ("Porcentaje _Y");
    gtk_box_pack_start (GTK_BOX (hbox), label, FALSE, FALSE, 0);
    spin_y = gtk_spin_button_new_with_range (MIN_DEGREE, MAX_DEGREE, 1.0);
    gtk_spin_button_set_value ((GtkSpinButton*)spin_y, 0);
    gtk_box_pack_start (GTK_BOX (hbox), spin_y, FALSE, FALSE, 0);
    gtk_label_set_mnemonic_widget (GTK_LABEL (label), spin_y);

    label = gtk_label_new_with_mnemonic ("_Interpolacion");
    gtk_box_pack_start (GTK_BOX (hbox), label, FALSE, FALSE, 0);
    spin_t = gtk_spin_button_new_with_range (0, 1, 1.0);
    gtk_spin_button_set_value ((GtkSpinButton*)spin_t, 0);
    gtk_box_pack_start (GTK_BOX (hbox), spin_t, FALSE, FALSE, 0);
    gtk_label_set_mnemonic_widget (GTK_LABEL (label), spin_t);
    
    frame = gtk_frame_new ("Interpolacion -- 1: bilineal -- 0 vec. max cercano ");
    gtk_container_set_border_width (GTK_CONTAINER(frame), 8);
    gtk_container_add (GTK_CONTAINER(frame), hbox);
    
    gtk_widget_show_all (frame);
    gtk_box_pack_start (GTK_BOX(GTK_DIALOG(dialog)->vbox), frame, FALSE, FALSE, 0);
    response = gtk_dialog_run (GTK_DIALOG(dialog));
    if (response == GTK_RESPONSE_OK)
    {
        *percent_x = gtk_spin_button_get_value (GTK_SPIN_BUTTON(spin_x));
        *percent_y = gtk_spin_button_get_value (GTK_SPIN_BUTTON(spin_y));
	*type = gtk_spin_button_get_value (GTK_SPIN_BUTTON(spin_t));
	response = 1;
    }
    else
	response = 0;
    gtk_widget_destroy (dialog);
    return response;
}
/* -------------------------------------------------------------------------------------------- */

/* **********************************************************************
 * Una tabla con dos spins buttons
 * ********************************************************************** */
GtkWidget *frame_two_spin_buttons (const char* str, int minx, int maxx, int miny, int maxy, GtkWidget **spin_x, GtkWidget **spin_y)
{
    GtkWidget *table, *label;
    GtkWidget *frame;

    table = gtk_table_new (2, 2,FALSE);
    gtk_container_set_border_width (GTK_CONTAINER(table), 8);
    gtk_table_set_row_spacings (GTK_TABLE(table), 4);
    gtk_table_set_col_spacings (GTK_TABLE(table), 4);
    
    label = gtk_label_new_with_mnemonic ("_x");
    gtk_table_attach_defaults (GTK_TABLE(table), label, 0,1,0,1);
    *spin_x = gtk_spin_button_new_with_range (minx, maxx,1.0);
    gtk_table_attach_defaults (GTK_TABLE (table), *spin_x, 1, 2, 0, 1);
    gtk_label_set_mnemonic_widget (GTK_LABEL (label), *spin_x);

    label = gtk_label_new_with_mnemonic ("_y");
    gtk_table_attach_defaults (GTK_TABLE(table), label, 0,1,1,2);
    *spin_y = gtk_spin_button_new_with_range (miny, maxy, 1.0);
    gtk_table_attach_defaults (GTK_TABLE (table), *spin_y, 1, 2, 1, 2);
    gtk_label_set_mnemonic_widget (GTK_LABEL (label), *spin_y);

    frame = gtk_frame_new (str);
    gtk_container_set_border_width (GTK_CONTAINER(frame), 8);
    gtk_container_add (GTK_CONTAINER(frame), table);

    return frame;
}
/* **********************************************************************
 * ruido gauss
 * ********************************************************************** */
GtkWidget* frame_gauss (const char* str , int width, int height,  
	GtkWidget **spin_w, GtkWidget  **spin_h, GtkWidget  **spin_m, GtkWidget **spin_s, GtkWidget **spin_n1, GtkWidget **spin_n2)
{
#define min_gauss	-255
#define max_gauss	+255
    GtkWidget *label;
    GtkWidget *table;
    GtkWidget *frame;
    
    table = gtk_table_new (3,4,FALSE);
    gtk_container_set_border_width (GTK_CONTAINER(table), 8);
    gtk_table_set_row_spacings (GTK_TABLE(table), 4);
    gtk_table_set_col_spacings (GTK_TABLE(table), 4);
   
    /* primera fila */
    label = gtk_label_new_with_mnemonic ("width");
    gtk_table_attach_defaults (GTK_TABLE(table), label, 0,1,0,1);
    *spin_w = gtk_spin_button_new_with_range (1, width,1.0);
    gtk_spin_button_set_value ((GtkSpinButton*)*spin_w, width);
    gtk_table_attach_defaults (GTK_TABLE (table), *spin_w, 1, 2, 0, 1);
    gtk_label_set_mnemonic_widget (GTK_LABEL (label), *spin_w);
    
    label = gtk_label_new_with_mnemonic ("height");
    gtk_table_attach_defaults (GTK_TABLE(table), label, 2,3,0,1);
    *spin_h = gtk_spin_button_new_with_range (1, height ,1.0);
    gtk_spin_button_set_value ((GtkSpinButton*)*spin_h, height);
    gtk_table_attach_defaults (GTK_TABLE (table), *spin_h, 3,4, 0, 1);
    gtk_label_set_mnemonic_widget (GTK_LABEL (label), *spin_h);
    
    /* segunda fila */
    label = gtk_label_new_with_mnemonic ("media");
    gtk_table_attach_defaults (GTK_TABLE(table), label, 0,1,1,2);
    *spin_m = gtk_spin_button_new_with_range (min_gauss, max_gauss,1.0);
    gtk_spin_button_set_value ((GtkSpinButton*)*spin_m, 0);
    gtk_table_attach_defaults (GTK_TABLE (table), *spin_m, 1, 2, 1,2);
    gtk_label_set_mnemonic_widget (GTK_LABEL (label), *spin_m);
    
    label = gtk_label_new_with_mnemonic ("desviacion");
    gtk_table_attach_defaults (GTK_TABLE(table), label, 2,3,1,2);
    *spin_s = gtk_spin_button_new_with_range (min_gauss, max_gauss,0.1);
    gtk_spin_button_set_value ((GtkSpinButton*)*spin_s, 1);
    gtk_table_attach_defaults (GTK_TABLE (table), *spin_s, 3,4, 1,2);
    gtk_label_set_mnemonic_widget (GTK_LABEL (label), *spin_s);

    /* tercera fila */
    label = gtk_label_new_with_mnemonic ("n1");
    gtk_table_attach_defaults (GTK_TABLE(table), label, 0,1,2,3);
    *spin_n1 = gtk_spin_button_new_with_range (min_gauss, max_gauss,1.0);
    gtk_spin_button_set_value ((GtkSpinButton*)*spin_n1, -1);
    gtk_table_attach_defaults (GTK_TABLE (table), *spin_n1, 1, 2, 2,3);
    gtk_label_set_mnemonic_widget (GTK_LABEL (label), *spin_n1);
    
    label = gtk_label_new_with_mnemonic ("n2");
    gtk_table_attach_defaults (GTK_TABLE(table), label, 2,3,2,3);
    *spin_n2 = gtk_spin_button_new_with_range (min_gauss, max_gauss,1.0);
    gtk_spin_button_set_value ((GtkSpinButton*)*spin_n2, 1);
    gtk_table_attach_defaults (GTK_TABLE (table), *spin_n2, 3,4, 2,3);
    gtk_label_set_mnemonic_widget (GTK_LABEL (label), *spin_n2);
    
    frame = gtk_frame_new (str);
    gtk_container_set_border_width (GTK_CONTAINER(frame), 8);
    gtk_container_add (GTK_CONTAINER(frame), table);

    return frame;
}

/* --------------------------------------------------------------------------------------------------------------- */

/* ***********************
 * spinx
 * ********************** */
void on_spinx_lineal_value_changed (GtkSpinButton *spin_x, gpointer data)
{
    lineal_t *lineal_v = (lineal_t*)data;
    int* points = lineal_v->points;
    GtkWidget *spin_y = lineal_v->spin_y;
    int x;
   
    x = gtk_spin_button_get_value_as_int (GTK_SPIN_BUTTON(spin_x));
    /* Se ha modificado el spinx x por tanto:
     * Asignamos a spin_y el valor que hay en points[x] */
    gtk_spin_button_set_value (GTK_SPIN_BUTTON(spin_y), points[x]);
}
/* ***********************
 * spiny
 * ********************** */
void on_spiny_lineal_value_changed (GtkSpinButton *spin_y, gpointer data)
{
    lineal_t *lineal_v = (lineal_t*)data;
    GdkPixmap* pixmap = lineal_v->pixmap;
    int* points = lineal_v->points;
    GtkWidget *spin_x = lineal_v->spin_x;
    int y;
    int x;
    GtkWidget *darea = lineal_v->darea;
    	
    y = gtk_spin_button_get_value_as_int (GTK_SPIN_BUTTON(spin_y));
    x = gtk_spin_button_get_value_as_int (GTK_SPIN_BUTTON(spin_x));
    if (points[x] != y)		/* Se ha puesto un nuevo valor */
    {
	if (((x == 0) || (x == 255)) && (y == -1))	/* points[0] y points[255] deben estar definidos, no dejamos poner un -1 */
	    gtk_spin_button_set_value (GTK_SPIN_BUTTON(spin_y), points[x]);
	
	else        					/* Actualizamos points[x] */
	{
	    if (points[x] != -1 && y == -1) 
		lineal_v->n--;
	    else if (points[x] == -1 && y != -1) 
		lineal_v->n++;
	    points[x] = y;
	    draw_lineal (darea, pixmap, points, lineal_v->n);
	}
    }
}
/* **********************************************************************
 * Pintamos sobre el pixmap los tramos que se corresponden con el ajuste
 * lineal
 * ********************************************************************** */
void draw_lineal (GtkWidget *darea, GdkPixmap *pixmap, int *points, int n)
{
    static GdkRectangle rec = {0, 0, 255, 255};
    GdkPoint *pts = (GdkPoint*) malloc (n * sizeof (GdkPoint));
    GdkGC *gc = gdk_gc_new (pixmap);
    int i, j;

    for (j = 0, i = 0; i < 256; i++)
	if (points[i] != -1)
	{
	    pts[j].x = i;
	    pts[j++].y = 255-points[i];
	}
    gdk_draw_rectangle (pixmap, darea->style->white_gc, TRUE, 0, 0, 255, 255);
    gdk_draw_lines (pixmap, gc, pts, n);
    gdk_window_invalidate_rect (darea->window, &rec, FALSE);
}

/* ----------------------------------------------------------------------------------------------- */

void draw_cross_section (GdkPixmap *pixmap, int *value, int n)
{
    int width, height, y0, w, i, min = INFINITO, max = -INFINITO;
    GdkRectangle rec;
    GdkPoint *points;
    GdkGC *gc;
    GdkColor color;
    double pend;

    
    gdk_drawable_get_size (pixmap, &width, &height);
    rec = (GdkRectangle){0, 0, width, height};
    points = (GdkPoint*) malloc (n * sizeof (GdkPoint));
    w = width / n;	/* nmero de pixeles que separaran a dos puntos de la grafica */
    /* se busca el minimo y el maximo para ajustar los valores al alto del pixmap */
    for (i = 0; i < n; i++)
    {
	if (value[i] > max) max = value[i];
	if (value[i] < min) min = value[i];
    }
    pend = (double)height /  ((double)max - (double)min);
    for (i = 0; i < n; i++)
    {
	points[i].x = i * w;
	points[i].y = height - ROUND((double)(value[i] - min) * pend);
    }
    gc = gdk_gc_new (pixmap);
    color = (GdkColor){0, 0xffff, 0xffff, 0xffff};	//blanco
    gdk_gc_set_rgb_fg_color(gc, &color);
    gdk_draw_rectangle (pixmap, gc, TRUE, 0, 0, width, height);
    color = (GdkColor){0, 0xffff, 0, 0};		//rojo
    gdk_gc_set_rgb_fg_color (gc, &color);
    gdk_draw_lines (pixmap, gc, points, n);
    color = (GdkColor){0, 0, 0, 0};			//negro
    gdk_gc_set_rgb_fg_color (gc, &color);
    y0 = ROUND ((double)(-min) * pend);			// eje x
    gdk_draw_line (pixmap, gc, 0, y0, width-1, y0);

}

/* **********************************************************************
 * Introduce en el vbox un drawing area sobre el que se pintar un pixmap
 * que contiene la grfica asociada al perfil almacenado en value
 * n: numero de puntos del perfil
 * Los valores del perfil se escalarn a las dimensiones del pixmap
 * ********************************************************************** */
void draw_graph_cross_section (GtkWidget *vbox, char *str, int *value, int n, int height)
{
    GtkWidget *darea, *frame;
    GdkPixmap *pixmap;
    int width;
    gint depth;

    if (n > 400)	/* el largo de la grafica ser mas o menos de 400 o si el numero de puntos es mayor, del numero de puntos */
	width = n;
    else
	width = 400 / n * n;
    frame = gtk_frame_new (str);
    gtk_container_set_border_width (GTK_CONTAINER(frame), 8);
    darea = gtk_drawing_area_new ();
    gtk_widget_set_size_request (darea, width, height);
    gtk_container_add (GTK_CONTAINER(frame), darea);
    gtk_box_pack_start (GTK_BOX (vbox), frame, TRUE, TRUE, 0);
    gdk_window_get_geometry (gdk_get_default_root_window(), NULL, NULL, NULL, NULL, &depth);
    pixmap = gdk_pixmap_new (NULL, width, height, depth);
    draw_cross_section (pixmap, value, n);

    /* seales */
    g_signal_connect (G_OBJECT(darea), "expose_event", G_CALLBACK(on_pixmap_expose_event), pixmap);
    g_signal_connect (G_OBJECT(darea), "size_allocate", G_CALLBACK(on_prueba), NULL);
}

void on_prueba (GtkWidget *widget, GtkAllocation *allocation, gpointer data)
{
    g_print ("width. %d, height: %d\n", allocation->width, allocation->height);
}

/* xxxxxxxxxxxxxxxxxxxx TRANSFORMACIONES GEOMETRICAS xxxxxxxxxxxxxxxxxxxxxx */

/* **********************************************************************
 * Espejo horizontal
 * ********************************************************************** */
image_t *idp_image_new_vertical_mirror (image_t *image)
{
    guchar *buf, *bufptr;
    int i, j, c1, c2;

    buf = (guchar*) malloc(image->width * image->height * sizeof(guchar));
    for (i = 0; i <= image->width / 2; i++)
    {
	for (j = 0; j < image->height; j++) 
	{
	    c1 = j * image->width + i;
	    c2 = j * image->width + (image->width - 1 - i);
	    buf[c1] = image->buf[c2];
	    buf[c2] = image->buf[c1];
	}
    }
    return idp_image_new (buf,image->width,image->height,image->cmap->colors);
}

/* **********************************************************************
 * Espejo vertical
 * ********************************************************************** */
image_t *idp_image_new_horizontal_mirror (image_t *image)
{
    guchar *buf;
    guchar *bufptr_t;	/* ptr target */
    guchar *bufptr_s;	/* ptr source */
    int j;

    buf = (guchar*) malloc(image->width * image->height * sizeof(guchar));
   
    bufptr_t = buf;
    bufptr_s = image->buf + (image->height -1) * image->width;
    for (j = 0; j < image->height; j++)
    {
	memcpy (bufptr_t, bufptr_s, image->width * sizeof(guchar));
	bufptr_t += image->width;
	bufptr_s -= image->width;
    }
    return idp_image_new (buf,image->width,image->height,image->cmap->colors);
}

/* **********************************************************************
 * Rotacin de 90 grados 
 * ********************************************************************** */
image_t *idp_image_new_rotation_90 (image_t *image)
{
    guchar *bufptr_s;	/* ptr source */
    guchar *buf;
    int x, y;		/* coordenadas (x,y) imagen fuente */
    int dest;		/* ndece buffer destino */

    bufptr_s = image->buf;
    buf = (guchar*) malloc(image->width * image->height * sizeof(guchar));
    for (y = 0; y < image->height; y++)
	for (x = 1; x <= image->width; x++)
	{
	    buf[((image->width - x) * image->height) + y] = *bufptr_s;
	    ++bufptr_s;
	}
    return idp_image_new (buf,image->height,image->width,image->cmap->colors);
}

/* **********************************************************************
 * Redimensionamos la imagen (percent --> porcentaje)
 * type 
 * 	-> 0: interpolacion vecino ms prximo
 * 	-> <>0: interpolacion bilineal
 * ********************************************************************** */
image_t *idp_image_new_resize (image_t *image, int percent_x, int percent_y,
	char type)
{
    guchar *buf, *bufptr;
    double value_x = 1.0 + (double)percent_x / 100.0;
    double value_y = 1.0 + (double)percent_y / 100.0;
    int i, j;
    double x, y;
    /* dimensiones de la nueva imagen */
    int width = value_x * image->width;
    int height = value_y * image->height;
    /* para la interpolacin bilineal */
    corners_t corners;
    corners_grays_t cg;
    double p, q;
    
    buf = (guchar*) malloc(width * height * sizeof(guchar));
    bufptr = buf;
    for (j = 0; j < height; j++)
	for (i = 0; i < width; i++)
	{
	    x = (double)i / value_x;	/* transformacin inversa */
	    y = (double)j / value_y;

	    if ((int)x == image->width - 1 || (int)y == image->height - 1)
		*bufptr = GRAYS(image,(int)x,(int)y);
	    
	    /* interpolacion bilineal */
	    else if (type) {
		p = x - (double) ((int)x);
		q = y - (double) ((int)y);
		corners = CORNERS(x,y);
		cg = CORNERS_GRAYS(image,corners);
		*bufptr = I_BILINEAL(cg,p,q);
	    }
	    /* interpolacion vecino mas proximo */
	    else
		*bufptr = GRAYS(image,ROUND(x),ROUND(y));
	    ++bufptr;
	}
    return idp_image_new (buf,width,height,NULL);
}

/* **********************************************************************
 * Rotamos la imagen "degree" grados
 * type:
 * 	-> 0: interpolacion vecino ms prximo
 * 	-> <>0: interpolacion bilineal
 * inverse:
 * 	-> 0: transformacin inversa (X',Y') -> (X,Y)
 * 	-> 1: transformacin directa (X,Y) -> (X',Y'), solo como curiosidat
 * ********************************************************************** */
image_t* idp_image_new_rotation (image_t *image, int degree, char type, char direct)
{
    guchar *buf, *bufptr;
    unsigned int *hist_abs;
    double v_cos, v_sin;
    double x_min_cos, x_max_cos, y_min_cos, y_max_cos;
    double x_min_sin, x_max_sin, y_min_sin, y_max_sin;
    double max_x, max_y, min_x, min_y;
    int width, height;
    double x, y;	/* (x,y) coord imagen original */
    double v, w;	/* (v,w) coord imagen destino (valores + y -)  */
    int i, j; 		/* (i,j) coord imagen destino (solo valores +) */
    corners_t corners;	/* para la interpolacin bilineal */
    corners_grays_t cg;
    double p, q;
    
    v_cos = cos ((double)degree * PI / 180.0);
    v_sin = sin ((double)degree * PI / 180.0);
    
    /* Transformacin inversa (X',Y') -> (X, Y), siendo (X',Y') los puntos
     * de la nueva imagen y (X,Y) los puntos de la imagen original:
     * La frmula que pasa de (X, Y) -> (X', Y') es:
     * x' = x * cos + y * sin
     * y' = y * cos - y * sin
     * 
     * */

    /* Buscamos los valores mnimo y mximos que tomar (X',Y') para 
     * calcular el tamao de la nueva imagen:
     * 0 <= x < image->widht
     * 0 <= y < image->height
     * MAX (x') = MAX (x cos + y sin  = MAX (x cos + MAX (y sin) =
     * 				      = x_max_cos  + x_max_sin
     * El resto de combinaciones es fcil de deducir.
     * */
    if (v_cos < 0.0) {
	x_min_cos = (double)(image->width-1) * v_cos;
	x_max_cos = 0.0;
	y_min_cos = (double)(image->height-1) * v_cos;
	y_max_cos = 0.0;
    }
    else {
	x_max_cos = (double)(image->width-1)  * v_cos;
	x_min_cos = 0.0;
	y_max_cos = (double)(image->height-1)  * v_cos;
	y_min_cos = 0.0;
    }
    if (v_sin < 0.0) {
	x_min_sin = (double)(image->height-1)  * v_sin;
	x_max_sin = 0.0;
	y_min_sin = (double)(image->width-1) * v_sin;
	y_max_sin = 0.0;
    }
    else {
	x_max_sin = (double)(image->height-1) * v_sin;
	x_min_sin = 0.0;
	y_max_sin = (double)(image->width-1) * v_sin;
	y_min_sin = 0.0;
    }
    max_x = x_max_cos + x_max_sin;
    max_y = y_max_cos - y_min_sin;
    min_x = x_min_cos + x_min_sin;
    min_y = y_min_cos - y_max_sin;

    width  = abs (ROUND(max_x - min_x)) + 1;
    height = abs (ROUND(max_y - min_y)) + 1;

    buf = (guchar*) calloc (width * height , sizeof(guchar));
    bufptr = buf;

    hist_abs = (unsigned int*) calloc (256, sizeof (unsigned int));

    /* Solo como curiosidad. No tiene valor prctico ----------------------- 
     * El histograma que genera tampo tiene ningn valor representativo ya que
     * es el mismo (exacto) del de la imagen de partido cuando no dibiera ser
     * as ya que hay huecos en la nueva imagen sin definir, mientras que otros
     * puntos son redefinidos porque (X,Y) -> (X',Y') no es nica.
     * Transformacin directa: Veamos que ocurre si transformamos los puntos
     * (X,Y) -> (X',Y').
     * Ocurre que quedan puntos dentro de la nueva imagen sin definir. Aparece
     * por tantos "huecos" negros */
    if (direct) {
	bufptr = image->buf;
	for (j = 0; j < image->height; j++)
	    for (i = 0; i < image->width; i++)
	    {
		x = (double)i * v_cos + (double)j * v_sin - min_x;
		y = (double)j * v_cos - (double)i * v_sin - min_y;
		buf[ROUND(y) * width + ROUND(x)] = *bufptr;
		hist_abs[*bufptr]++;
		++bufptr;
	    }
	return idp_image_new_with_hist_abs (buf,width,height,NULL, hist_abs);
    }
    /* --------------------------------------------------------------------- */
    /* Partiendo de (X,Y) -> (X',Y') se obtiene que las funciones para pasar
     * (X',Y') -> (X, Y) son:
     * x = x' cos - y' sin
     * y = y' cos + x' sin
     * */
    /* Para todos los puntos de la nueva imagen */
    for (j = 0, w = min_y; j < height; j++, w+=1.0)
	for (i = 0, v = min_x; i < width; i++, v+=1.0)
	{
	    /* Punto en la imagen original */
	    x = v * v_cos - w * v_sin;
	    y = w * v_cos + v * v_sin;

	    /* Si est dentro del rango de la imagen (sino --> color 0) */
	    if (((int)x >= 0 && (int)x < image->width) &&
		((int)y >= 0 && (int)y < image->height)) {

		/* borde derecho o inferior de la imagen original */
		if ((int)x == image->width - 1 || (int)y == image->height - 1) {
		    *bufptr = GRAYS(image,(int)x,(int)y);
		    hist_abs[*bufptr]++;
		}

    		/* Si type -> interpolacin bilineal */
		else if (type) {
		    p = x - (double) ((int)x);
		    q = y - (double) ((int)y);
		    corners = CORNERS(x,y);
		    cg = CORNERS_GRAYS(image,corners);
		    *bufptr = I_BILINEAL(cg,p,q);
		    hist_abs[*bufptr]++;
		}
		/* sino: interpolacion vecino mas proximo */
		else {
		    *bufptr = GRAYS(image,ROUND(x),ROUND(y));
		    hist_abs[*bufptr]++;
		}
	    }
	    ++bufptr;
	}
    return idp_image_new_with_hist_abs (buf,width,height,NULL, hist_abs);
}
